#include "Convolutions.h"
#include "cuba_int.h"

namespace CuTe {
namespace NS_Convolutions {

using NS_CUBA_integration::Cuba_Params;

ostream& _Convolution_Interface::_V_to_stream(ostream &os, const vector<double> &V, const int nf){
	os	<<"[ ";
	int i(-nf);
	for(;i<nf;++i){
		os	<<V[NFMAX+i]<<" , ";
	}
	os	<<V[NFMAX+i]<<" ]";

	return os;
}

void _Convolution_Interface::_update_S(const vector<double> &V, double &S, double &Sq, double &Sqb, const int nf){
	Sq	= 0;
	Sqb	= 0;
	for(size_t i=1; i<=nf; ++i){
		Sq	+= V[NFMAX+i];
		Sqb	+= V[NFMAX-i];
	}
	S	= Sq + Sqb;
}

void _Convolution_Interface::set_nf(const size_t nf){
	this->nf	= nf;
	_set_nf();
}

double _Convolution_Interface::get_fxi(const int lha_flavor)const{
	if(abs(lha_flavor)>nf){
		cerr << "_Convolution_Interface::get_f1("<<lha_flavor<<") nf = "<<nf<<endl;
		return 0.;
	}
	return Vfxi[NFMAX+lha_flavor];
}

double _Convolution_Interface::get_Sxiq(const int lha_flavor)const{
	if(abs(lha_flavor)>nf){
		cerr << "_Convolution_Interface::get_Sxiq("<<lha_flavor<<") nf = "<<nf<<endl;
		return 0.;
	}
	return ( (lha_flavor>0) ? (Sxiq) : (Sxiqb));
}

double _Convolution_Interface::get_Sxiqb(const int lha_flavor)const{
	if(abs(lha_flavor)>nf){
		cerr << "_Convolution_Interface::get_Sxiqb("<<lha_flavor<<") nf = "<<nf<<endl;
		return 0.;
	}
	return ( (lha_flavor>0) ? (Sxiqb) : (Sxiq));
}

ostream& _Convolution_Interface::to_stream(ostream &os, const int ntabs)const{
	string tab("");
	for(int i=0; i<ntabs; ++i) tab += "\t";

	os	<<tab<<"_Convolution_Interface:"<<endl;
	os	<<tab<<"\t"<<"nf = "<<nf<<"\t"<<"xi = "<<xi<<endl;
	os	<<tab<<"\t"<<"fxi = ";_V_to_stream(os,Vfxi,nf)<<endl;
	os	<<tab<<"\t"<<"Sxi = "<<Sxi<<"\t"<<"Sxiq = "<<Sxiq<<"\t"<<"Sxiqb = "<<Sxiqb<<endl;

	return os;
}

ostream& operator<<(ostream& os, const _Convolution_Interface &CI){
	return CI._V_to_stream(os,CI.get_Vfxi(),CI.get_nf());
}

void _Convolution_Integrand::_set_nf(){
	for(size_t i=nf+1; i<=NFMAX; ++i){
		Vfxi[NFMAX+i]	= 0.;
		Vfxi[NFMAX-i]	= 0.;
		Vfz[NFMAX+i]	= 0.;
		Vfz[NFMAX-i]	= 0.;
	}
}

void _Convolution_Integrand::set_nf(const size_t nf){
	f->set_nf(nf);
	_Convolution_Interface::set_nf(nf);
}

void _Convolution_Integrand::update_xi_z(const double xi, const double xx[]){
//	cout	<<"_Convolution_Integrand::update_xi_z("<<xi<<","<<*xx<<")"<<endl;
	this->xi	= xi;
	Jac1	= 1. -xi;
	z1	= xi	+ (*xx)*Jac1;
	f->_update_xi1_xi2_z(xi,xi/z1,++xx);
	_update_V(Vfxi,xi,z1,Jac1);
	_update_S(Vfxi,Sxi,Sxiq,Sxiqb,nf);
}

void _Convolution_Integrand::_update_xi1_xi2_z(const double xi1, const double xi2, const double xx[]){
//	cout	<<"_Convolution_Integrand::update_xi_z("<<xi1<<","<<xi2<<","<<*xx<<")"<<endl;
	update_xi_z(xi1,xx);

	this->xi2	= xi2;
	Jac2	= 1. -xi2;
	z2	= xi2	+ (*xx)*Jac2;
	f->_update_xi1_xi2_z(xi2,xi2/z2,++xx);
	_update_V(Vfz ,xi2,z2,Jac2);
	_update_S(Vfz,Sz,Szq,Szqb,nf);
}

double _Convolution_Integrand::get_fz(const int lha_flavor)const{
//	cout	<<"_Convolution_Integrand::get_fz()"<<endl;
	if(abs(lha_flavor)>nf){
		cerr << "_Convolution_Integrand::get_fz("<<lha_flavor<<") nf = "<<nf<<endl;
		return 0.;
	}
	return Vfz[NFMAX+lha_flavor];
}

double _Convolution_Integrand::get_Szq(const int lha_flavor)const{
	if(abs(lha_flavor)>nf){
		cerr << "_Convolution_Integrand::get_Szq("<<lha_flavor<<") nf = "<<nf<<endl;
		return 0.;
	}
	return ( (lha_flavor>0) ? (Szq) : (Szqb));
}

double _Convolution_Integrand::get_Szqb(const int lha_flavor)const{
	if(abs(lha_flavor)>nf){
		cerr << "_Convolution_Integrand::get_Szqb("<<lha_flavor<<") nf = "<<nf<<endl;
		return 0.;
	}
	return ( (lha_flavor>0) ? (Szqb) : (Szq));
}

ostream& _Convolution_Integrand::to_stream(ostream &os, const int ntabs)const{
	string tab("");
	for(int i=0; i<ntabs; ++i) tab += "\t";

	os	<<tab<<"_Convolution_Integrand:"<<endl;
	os	<<tab<<"\t"<<"dim = "<<get_dim()<<"\t"<<"nf = "<<nf<<endl;
	os	<<tab<<"\t"<<"xi1 = "<<xi <<"\t"<<"z1 = "<<z1<<endl;
	os	<<tab<<"\t"<<"fxi1 = ";_V_to_stream(os,Vfxi,nf)<<endl;
	os	<<tab<<"\t"<<"xi2 = "<<xi2<<"\t"<<"z2 = "<<z2<<endl;
	os	<<tab<<"\t"<<"fxi2 = ";_V_to_stream(os,Vfz,nf)<<endl;

	return os;
}

int _Convolution_Integrand::get_dim(){
	dim	= f->get_dim();
	if(dim>=0) ++dim;
	return dim;
}

int _Convolution_Integrand::get_dim()const{
	return f->get_dim()+1;
}

void Convolution_Function::update_xi_z(const double xi, const double xx[]){
//	cout	<<"Convolution_Function::update_xi_z("<<xi<<","<<*xx<<")"<<endl;
	this->xi	= xi;
	Jac1		= 1. -xi;
	z1			= xi	+ (*xx)*Jac1;

	_update_V(Vfxi,xi,z1,Jac1);
	_update_S(Vfxi,Sxi,Sxiq,Sxiqb,nf);
}

void Convolution_Function::_update_xi1_xi2_z(const double xi, const double z, const double xx[]){
//	cout	<<"Convolution_Function::_update_xi1_xi2_z("<<xi<<","<<z<<")"<<endl;
	this->xi	= xi;
	this->z1	= z;
	Jac1		= 1.;
	_update_V(Vfxi,xi,1.,1.);
	_update_S(Vfxi,Sxi,Sxiq,Sxiqb,nf);
	_update_V(Vfz ,z ,1.,1. );
	_update_S(Vfz ,Sz,Szq,Szqb,nf);
}

void Convolution_Function::_update_V(vector<double> &V, const double xi, const double z, const double Jac){
	for(int fl=-nf; fl<=+nf; ++fl){
		V[NFMAX+fl]	= Jac*g(xi/z,fl,params);
	}
}


void Convolution_Function::set_nf(const size_t nf){
	_Convolution_Interface::set_nf(nf);
}



void Convolution_Function::init_function(CI_Function g, void *params){
	dim	= 0;
	this->g			= g;
	this->params	= params;
}

ostream& Convolution_Function::to_stream(ostream &os, const int ntabs)const{
	string tab("");
	for(int i=0; i<ntabs; ++i) tab += "\t";

	os	<<tab<<"Convolution_Function:"<<endl;
	os	<<tab<<"\t"<<" xi  = "<<xi<<"\t"<<"f( xi ) = ";_V_to_stream(cout,Vfxi,nf)<<endl;
	os	<<tab<<"\t"<<"xi/z = "<<z1<<"\t"<<"f(xi/z) = ";_V_to_stream(cout,Vfz ,nf)<<endl;

	return os;
}

int Convolution_Integral::_integrand(double ff[],const double xx[]){
	f->update_xi_z(xi,xx);
//	f->to_stream(cout,1);
	for(int fl=(-nf); fl<=(+nf); ++fl){
		(*(ff++))	=f->get_fxi(fl);
	}
	return 0;
}

int Convolution_Integral::_cuba_integrand(const int *ndim, const double xx[],const int *ncomp, double ff[], void *userdata){
	Convolution_Integral *CI = static_cast<Convolution_Integral*>(userdata);
	return CI->_integrand(ff,xx);
}


_Convolution_Integrand& Convolution_Integral::init_Integrand(_Convolution_Integrand *f){
	this->f	= f;
	return *f;
}

_Convolution_Integrand& Convolution_Integral::init_Integrand(_Convolution_Integrand *f, const Cuba_Params *cuba){
	delete this->cuba;
	this->cuba	= new Cuba_Params(*cuba);
	return init_Integrand(f);
}

void Convolution_Integral::_set_nf(){
	f->set_nf(nf);
	cuba->set(f->get_dim(),2*nf+1);
}

void Convolution_Integral::update(const double xi){
	this->xi		= xi;

	cuba->cube(_cuba_integrand,this);
//	cuba->test(2,_cuba_integrand,this);	//FIXME


	vector<double>::const_iterator it(cuba->integral.begin());
	vector<double>::const_iterator itE(cuba->error.begin());

	for(int fl=-nf; fl<=+nf; ++fl){
		Vfxi[NFMAX+fl]		= (*(it++));
		errors[NFMAX+fl]	= (*(itE++));
	}

	_update_S(Vfxi,Sxi,Sxiq,Sxiqb,nf);
}


Convolution_Integral::Convolution_Integral()
:_Convolution_Interface()
,f(),cuba(new Cuba_Params)
,errors(2*NFMAX+1,0.){
	cuba->epsrel	= 1e-6;
	cuba->routine	= NS_enums::NS_CUBA::CUBA_R_CUHRE;
	cuba->maxeval	= 1e6;
	cuba->set_ncores(4);
}

Convolution_Integral::Convolution_Integral(const Convolution_Integral& CI)
:_Convolution_Interface(CI)
,f(CI.f),cuba(new Cuba_Params(*(CI.cuba)))
,errors(CI.errors){

}

Convolution_Integral::~Convolution_Integral(){
	delete cuba;
}

Convolution_Integral& Convolution_Integral::operator=(const Convolution_Integral& CI){
	_Convolution_Interface::operator=(CI);
	f		= CI.f;
	delete cuba;
	cuba	= new Cuba_Params(*(CI.cuba));
	errors	= CI.errors;
	return *this;
}

ostream& Convolution_Integral::to_stream(ostream &os, const int ntabs)const{
	string tab("");
	for(int i=0; i<ntabs; ++i) tab += "\t";

	os	<<tab<<"Convolution_Integral:"<<endl;
	_Convolution_Interface::to_stream(os,ntabs+1);
	os	<<tab<<"\t"<<"errors =";_V_to_stream(cout,errors,nf)<<endl;
	os	<<tab<<"\t"<<"function *f ="<<f<<endl;
	if(f!=NULL) f->to_stream(os,ntabs+1);

	return os;
}

}  // namespace NS_Convolutions
}  // namespace CuTe
