#ifndef CUTE_H
#define CUTE_H

#include "config_CuTe.h"
#include "cuba_int.h"
#include "QDCS.h"
#include "CuTe_Options.h"
#include "Kinematics.h"
#include "Stepfunction.h"



namespace CuTe {

using NS_FO::FO_Kinematic_Paramete_Set;
using NS_CUBA_integration::Cuba_Params;
using NS_Kinematics::Kinematics;
using NS_QDCS::Quad_Differential_Crosssection;
using NS_Resumed::Res_Kinematic_Parameters;

//using NS_Polynom::Continious_Order_Reduction;


using NS_Utils::NS_pow::pow_2;


using namespace NS_enums::NS_RES;
using namespace NS_enums::NS_PC;
using namespace NS_enums::NS_RESULTS;

using std::cerr;
using std::cout;
using std::endl;
using std::vector;

//using NS_CROSSSECTION::Quad_Differential_Crosssection;
//using NS_Kinematics::Kinematics;

class CuTe_INIT_EXIT{
protected:
	string add;
public:
	CuTe_INIT_EXIT();
	CuTe_INIT_EXIT(const string& add);
	~CuTe_INIT_EXIT();
};





using NS_Stepfunction::TransitionFunction;


class Crosssection{
public:
	vector<double> results;
protected:

	Kinematics	Kin;
	Cuba_Params	cuba;

	RES res;
	FO_Kinematic_Paramete_Set *FPS;
	Res_Kinematic_Parameters *RPS;

	TransitionFunction transition_function;


	NS_CS_Params::PDFset pdfset;
	int N_pdf;

	NS_CS_Params::MU_star mu_star;
	double q_star;
	int N_mu;

	bool exp_errors;

	bool partial_results;
	int N_QDCS;	//= 1 + N_pdf + N_mu + exp_errors;
	int N_Centre;
	int N_Comp;

	Quad_Differential_Crosssection QDCS_central;
	vector<Quad_Differential_Crosssection> QDCS_mu;
	vector<Quad_Differential_Crosssection> QDCS_pdf;
	vector<Quad_Differential_Crosssection*> _QDCSs;
	Quad_Differential_Crosssection QDCS_exponent;


	vector<double> _results_mu_c_0_1;
	vector<double> _results_pdf;

	bool without_combined_result;

	template < RES thisres >
	inline void _fill_xx(const double xx[], double ff[]){
//		NS_Utils::DOUT dout("Crosssection::_fill_xx");
		vector<Quad_Differential_Crosssection*>::iterator it_qdcs(_QDCSs.begin());

		if( ( (FPS!=NULL)&&Kin.update_zzb_and_xixib_get_OK(xx) ) || Kin.update_zzb_get_OK(xx)){
			const double Jac_y	= Kin.Jac_y;

			if(RPS!=NULL) RPS->update_zz(&Kin);
			if(FPS!=NULL) FPS->update_xx(&Kin);

			if(partial_results){
				ff	= (*it_qdcs)->update_xx_set_f<thisres>(RPS,FPS,Jac_y,ff);
			}else{
				(*ff)	= (*it_qdcs)->update_xx<thisres>(RPS,FPS,Jac_y);
			}


			for(int i=1; i<N_QDCS; ++i){
				(*(++ff))	= (*(++it_qdcs))->update_xx<thisres>(RPS,FPS,Jac_y);

			}
		}else{
			(*ff)	= 0.;
			for(int i=1; i<N_Comp; ++i)	(*(++ff))	= 0.;
		}
	}


	template < RES thisres >
	inline void _fill_yxx(const double yxx[], double ff[]){
//		NS_Utils::DOUT dout("Crosssection::_fill_yxx");
		vector<Quad_Differential_Crosssection*>::iterator it_qdcs(_QDCSs.begin());

		if( ( (FPS!=NULL)&&Kin.update_y_zzb_and_xixib_get_OK(yxx)) || Kin.update_y_zzb_get_OK(yxx)){

			const double Jac_y	= Kin.Jac_y;

	//		dout()<<"y="<<Kin.y<<"\tJac_y="<<Jac_y<<endl;

			if(RPS!=NULL){
				RPS->set_y(&Kin);
				RPS->update_zz(&Kin);
			}
			if(FPS!=NULL){
				FPS->set_y(&Kin);
				FPS->update_xx(&Kin);
			}

			if(partial_results){
				ff	= (*it_qdcs)->update_y_xx_set_f<thisres>(RPS,FPS,Jac_y,ff);
			}else{
				(*ff)	= (*it_qdcs)->update_y_xx<thisres>(RPS,FPS,Jac_y);
			}

			for(int i=1; i<N_QDCS; ++i){
				(*(++ff))	= (*(++it_qdcs))->update_y_xx<thisres>(RPS,FPS,Jac_y);

			}
		}else{
			(*ff)	= 0.;
			for(int i=1; i<N_Comp; ++i)	(*(++ff))	= 0.;
		}
	}

	template < RES thisres >
	inline static int _cuba_int_yxx(const int *ndim, const double yxx[],const int *ncomp, double ff[], void *userdata){
//		NS_Utils::DOUT dout("Crosssection::_cuba_int_yxx");

		Crosssection* CS = static_cast<Crosssection*>(userdata);

		CS->_fill_yxx<thisres>(yxx,ff);

		return 0;
	}

	template < RES thisres >
	inline static int _cuba_int_xx(const int *ndim, const double xx[],const int *ncomp, double ff[], void *userdata){
//		NS_Utils::DOUT dout("Crosssection::_cuba_int_xx");

		Crosssection* CS = static_cast<Crosssection*>(userdata);

		CS->_fill_xx<thisres>(xx,ff);

		return 0;
	}

	void _set_qT_mu(const double mu);
	void _set_qT_y_mu(const double mu);

	template< RES thisres >
	inline bool _set_qT_mu_y_get_OK(const double qT, const double mu, const double y){
//		NS_Utils::DOUT dout("Crosssection::_set_qT_mu_y_get_OK","qT,mu,y");
		bool OK(Kin.set_qT_y_get_ok(qT,y));
		_set_qT_y_mu(mu);
		if(OK){
//			NS_Utils::DOUT dout("Crosssection::_set_qT_mu_y_get_OK","Kin.set_qT_y_get_ok(qT,y)=true");
			if(RPS!=NULL)	RPS->set_y(&Kin);
			if(FPS!=NULL)	FPS->set_y(&Kin);


			vector<Quad_Differential_Crosssection*>::iterator it_qdcs(_QDCSs.begin());

			(*it_qdcs)->set_y<thisres>(RPS,FPS);

			for(int i=1; i<N_QDCS; ++i){
				(*(++it_qdcs))->set_y<thisres>(RPS,FPS);
			}
			return true;
		}else{
			return false;
		}
	}

	void _set_results();

	vector<double>& _update(const int dim, NS_CUBA_integration::cuba_integrand_t integrand);

	void _init_params(const SOPS_CS& OP);

	vector<double>& _unset_results();

	vector<double>& update_qT_mu(const double qT, const double mu);
	vector<double>& update_qT_mu_y(const double qT, const double mu, const double y);
public:

	void init(const SOPS_CS& OP);


	inline vector<double>& update_qT(const double qT, const double qT_for_mu){
//		NS_Utils::DOUT dout("Crosssection::update_qT");
		return update_qT_mu(qT,mu_star(qT_for_mu));
	}

	inline vector<double>& update_qT(const double qT){
//		NS_Utils::DOUT dout("Crosssection::update_qT");
		return update_qT(qT,qT);
	}


	inline vector<double>& update_qT_y(const double qT, const double qT_for_mu, const double y){
		return update_qT_mu_y(qT,mu_star(qT_for_mu),y);
	}

	inline vector<double>& update_qT_y(const double qT, const double y){
		return update_qT_y(qT,qT,y);
	}

	inline double get_q_star()const{
		return q_star;
	}

	inline double get_alpha_s_MZ(const double MZ=DEFAULT_MZ)const{
		return pdfset.get_alpha_s_of(MZ);
	}


	Crosssection(const bool without_combined_result=false)
	:results()
	,res(RES_WRONG_STATEMENT)
	,RPS(NULL),FPS(NULL)
	,_QDCSs()
	,pdfset(),N_pdf(0)
	,mu_star(),q_star(),N_mu(0)
	,exp_errors(false)
	,partial_results(false)
	,N_QDCS(0),N_Centre(0),N_Comp(0)
	,QDCS_central()
	,QDCS_mu(),QDCS_pdf()
	,cuba()
	,without_combined_result(without_combined_result){

	}

	~Crosssection();


	std::vector<bool>& to_plot(std::vector<bool>& to_plot, const bool with_y=false)const;
	const std::vector<bool> to_plot(const bool with_y=false)const;
	std::ostream& FL_into_stream(std::ostream& os, const std::vector<bool>& to_plot, const int width)const;
	std::ostream& into_stream(std::ostream& os, const std::vector<bool>& to_plot, const int width, const int prec)const;


	inline void change_NPE(const double Lambda_NP, const NS_enums::NS_NP::CUTOFF cutoff){
		vector<Quad_Differential_Crosssection*>::iterator itr_qdcs(_QDCSs.begin());
		const vector<Quad_Differential_Crosssection*>::iterator end_qdcs(_QDCSs.end());
		while(itr_qdcs!=end_qdcs){
			(*(itr_qdcs++))->change_NPE(Lambda_NP,cutoff);
		}
	}

	inline void change_mustar(const SOPS_CS& OP){
		mu_star.init(OP,pdfset.as_central());
		//FIXME no change in scale-error possible

		NS_CS_Params::Q_star QS(pdfset.as_central(),OP);
		q_star	= QS();
	}

protected:

	inline void _change_PDF(const SOPS_CS& OP){
		NS_CS_Params::PDFset::iterator it_pdf(pdfset.begin());
		QDCS_central.change_PDF(*it_pdf);
		if(N_mu){
			for(int i=0; i<N_mu; ++i){
				QDCS_mu[i].change_PDF(*it_pdf);
			}
		}
		if(exp_errors){
			QDCS_exponent.change_PDF(*it_pdf);
		}
		if(N_pdf){
			for(int i=0; i<N_pdf; ++i){
				QDCS_pdf[i].change_PDF(*(++it_pdf));
			}
		}

		change_mustar(OP);
	}

public:

	inline void change_PDF(const SOPS_CS& OP){
		//FIXME no change in PDF_error possible
		if(static_cast<bool>(pdfset.N_error())!=OP.PDF.PDF_Error()){
			cerr	<< "Crosssection::change_PDF different PDF_Error values"<<endl;
			exit(1);
		}else{
			pdfset.change_PDF(OP);
			_change_PDF(OP);
		}
	}

	inline void change_PDF(const SOPS_CS& OP, LHAPDF::PDFSet *Set){
		//FIXME no change in PDF_error possible
		if(static_cast<bool>(pdfset.N_error())!=OP.PDF.PDF_Error()){
			cerr	<< "Crosssection::change_PDF different PDF_Error values"<<endl;
			exit(1);
		}else{
			pdfset.change_PDF(OP,Set);
			_change_PDF(OP);
		}
	}

	inline void change_PDF(const SOPS_CS& OP, NS_LHAFUNC::OWN_LHAPDF* own){
		//FIXME no change in PDF_error possible
		if(static_cast<bool>(pdfset.N_error())!=OP.PDF.PDF_Error()){
			cerr	<< "Crosssection::change_PDF different PDF_Error values"<<endl;
			exit(1);
		}else{
			pdfset.change_PDF(own);
			_change_PDF(OP);
		}
	}

};

}  // namespace CuTe

#endif
