#ifndef IBAR_G_FUNCTIONS_H
#define IBAR_G_FUNCTIONS_H

#include "Ibar_G_Parameter.h"
#include "Ibar_G_Constants.h"
#include "polynom.h"


namespace CuTe{
namespace NS_Ibar{

using namespace NS_APSF;
typedef NS_Polynom::Pol<double> Pol_l;
typedef NS_Polynom::Pol< Pol_l > Pol_a_l;


using namespace NS_Ibar_G_Constants;

using NS_Ibar::Ibar_G_Parameter;
using NS_PlusDists::PD;

template< e_P j, int a, int l, int e >
inline double Ibar_G(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return 0.;
}

template< e_P j, int a, int l, int e >
inline double Ibar_G2(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return 0.;
}


template< e_P j >
inline Pol_a_l& Ibar_G_pol_set(Pol_a_l &P, double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	const int  ord(p.order);
	const bool imp(p.improved);
		
	P[0][0] = Ibar_G<j,0,0,0>(fz,f1,p,Nf);
	
	if(ord>=1){
		
		P[1][1] = Ibar_G<j,1,1,1>(fz,f1,p,Nf);
		
		if( (!imp)||(ord>=2) ){
			
			P[1][0] = Ibar_G<j,1,0,2>(fz,f1,p,Nf);
			
		}	//e==2
		
		
		if(ord>=2){
			
			P[2][2] = Ibar_G<j,2,2,2>(fz,f1,p,Nf);
			
			if( (!imp)||(ord>=3) ){
				
				P[2][1] = Ibar_G<j,2,1,3>(fz,f1,p,Nf);
				
				if( (!imp)||(ord>=4) ){
					
					P[2][0] = Ibar_G<j,2,0,4>(fz,f1,p,Nf);
					
				}	//e==4
				
			}	//e==3
			
			
			if(ord>=3){
				
				P[3][3] = Ibar_G<j,3,3,3>(fz,f1,p,Nf);
				
				if( (!imp)||(ord>=4) ){
					
					P[3][2] = Ibar_G<j,3,2,4>(fz,f1,p,Nf);
					
					if( (!imp)||(ord>=5) ){
						
						P[3][1] = Ibar_G<j,3,1,5>(fz,f1,p,Nf);
						
					}	//e==5
					
				}	//e==4
				
				
				if(ord>=4){
					
					P[4][4] = Ibar_G<j,4,4,4>(fz,f1,p,Nf);
					
					if( (!imp)||(ord>=5) ){
						
						P[4][3] = Ibar_G<j,4,3,5>(fz,f1,p,Nf);
						
					}	//e==5
					
					
					if(ord>=5){
						
						P[5][5] = Ibar_G<j,5,5,5>(fz,f1,p,Nf);
						
					}	//a==5
				}	//a==4
			}	//a==3
		}	//a==2
	}	//a==1
	return P;
}

template< e_P j >
inline Pol_a_l& Ibar_G_pol_add(Pol_a_l &P, double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	const int  ord(p.order);
	const bool imp(p.improved);
		
	P[0][0] += Ibar_G<j,0,0,0>(fz,f1,p,Nf);
	
	if(ord>=1){
		
		P[1][1] += Ibar_G<j,1,1,1>(fz,f1,p,Nf);
		
		if( (!imp)||(ord>=2) ){
			
			P[1][0] += Ibar_G<j,1,0,2>(fz,f1,p,Nf);
			
		}	//e==2
		
		
		if(ord>=2){
			
			P[2][2] += Ibar_G<j,2,2,2>(fz,f1,p,Nf);
			
			if( (!imp)||(ord>=3) ){
				
				P[2][1] += Ibar_G<j,2,1,3>(fz,f1,p,Nf);
				
				if( (!imp)||(ord>=4) ){
					
					P[2][0] += Ibar_G<j,2,0,4>(fz,f1,p,Nf);
					
				}	//e==4
				
			}	//e==3
			
			
			if(ord>=3){
				
				P[3][3] += Ibar_G<j,3,3,3>(fz,f1,p,Nf);
				
				if( (!imp)||(ord>=4) ){
					
					P[3][2] += Ibar_G<j,3,2,4>(fz,f1,p,Nf);
					
					if( (!imp)||(ord>=5) ){
						
						P[3][1] += Ibar_G<j,3,1,5>(fz,f1,p,Nf);
						
					}	//e==5
					
				}	//e==4
				
				
				if(ord>=4){
					
					P[4][4] += Ibar_G<j,4,4,4>(fz,f1,p,Nf);
					
					if( (!imp)||(ord>=5) ){
						
						P[4][3] += Ibar_G<j,4,3,5>(fz,f1,p,Nf);
						
					}	//e==5
					
					
					if(ord>=5){
						
						P[5][5] += Ibar_G<j,5,5,5>(fz,f1,p,Nf);
						
					}	//a==5
				}	//a==4
			}	//a==3
		}	//a==2
	}	//a==1
	return P;
}

template< e_P j >
inline Pol_a_l& Ibar_G2_pol_set(Pol_a_l &P, double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	const int  ord(p.order);
	const bool imp(p.improved);
		
	P[0][0] = Ibar_G2<j,0,0,0>(fz,f1,p,Nf);
	
	if(ord>=1){
		
		P[1][1] = Ibar_G2<j,1,1,1>(fz,f1,p,Nf);
		
		if( (!imp)||(ord>=2) ){
			
			P[1][0] = Ibar_G2<j,1,0,2>(fz,f1,p,Nf);
			
		}	//e==2
		
		
		if(ord>=2){
			
			P[2][2] = Ibar_G2<j,2,2,2>(fz,f1,p,Nf);
			
			if( (!imp)||(ord>=3) ){
				
				P[2][1] = Ibar_G2<j,2,1,3>(fz,f1,p,Nf);
				
				if( (!imp)||(ord>=4) ){
					
					P[2][0] = Ibar_G2<j,2,0,4>(fz,f1,p,Nf);
					
				}	//e==4
				
			}	//e==3
			
			
			if(ord>=3){
				
				P[3][3] = Ibar_G2<j,3,3,3>(fz,f1,p,Nf);
				
				if( (!imp)||(ord>=4) ){
					
					P[3][2] = Ibar_G2<j,3,2,4>(fz,f1,p,Nf);
					
					if( (!imp)||(ord>=5) ){
						
						P[3][1] = Ibar_G2<j,3,1,5>(fz,f1,p,Nf);
						
					}	//e==5
					
				}	//e==4
				
				
				if(ord>=4){
					
					P[4][4] = Ibar_G2<j,4,4,4>(fz,f1,p,Nf);
					
					if( (!imp)||(ord>=5) ){
						
						P[4][3] = Ibar_G2<j,4,3,5>(fz,f1,p,Nf);
						
					}	//e==5
					
					
					if(ord>=5){
						
						P[5][5] = Ibar_G2<j,5,5,5>(fz,f1,p,Nf);
						
					}	//a==5
				}	//a==4
			}	//a==3
		}	//a==2
	}	//a==1
	return P;
}

template< e_P j >
inline Pol_a_l& Ibar_G2_pol_add(Pol_a_l &P, double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	const int  ord(p.order);
	const bool imp(p.improved);
		
	P[0][0] += Ibar_G2<j,0,0,0>(fz,f1,p,Nf);
	
	if(ord>=1){
		
		P[1][1] += Ibar_G2<j,1,1,1>(fz,f1,p,Nf);
		
		if( (!imp)||(ord>=2) ){
			
			P[1][0] += Ibar_G2<j,1,0,2>(fz,f1,p,Nf);
			
		}	//e==2
		
		
		if(ord>=2){
			
			P[2][2] += Ibar_G2<j,2,2,2>(fz,f1,p,Nf);
			
			if( (!imp)||(ord>=3) ){
				
				P[2][1] += Ibar_G2<j,2,1,3>(fz,f1,p,Nf);
				
				if( (!imp)||(ord>=4) ){
					
					P[2][0] += Ibar_G2<j,2,0,4>(fz,f1,p,Nf);
					
				}	//e==4
				
			}	//e==3
			
			
			if(ord>=3){
				
				P[3][3] += Ibar_G2<j,3,3,3>(fz,f1,p,Nf);
				
				if( (!imp)||(ord>=4) ){
					
					P[3][2] += Ibar_G2<j,3,2,4>(fz,f1,p,Nf);
					
					if( (!imp)||(ord>=5) ){
						
						P[3][1] += Ibar_G2<j,3,1,5>(fz,f1,p,Nf);
						
					}	//e==5
					
				}	//e==4
				
				
				if(ord>=4){
					
					P[4][4] += Ibar_G2<j,4,4,4>(fz,f1,p,Nf);
					
					if( (!imp)||(ord>=5) ){
						
						P[4][3] += Ibar_G2<j,4,3,5>(fz,f1,p,Nf);
						
					}	//e==5
					
					
					if(ord>=5){
						
						P[5][5] += Ibar_G2<j,5,5,5>(fz,f1,p,Nf);
						
					}	//a==5
				}	//a==4
			}	//a==3
		}	//a==2
	}	//a==1
	return P;
}



//*****************************
//********   a = 0   ********
//*****************************

//**********************************************************
//********   l = 0   ****************   e = 0   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,0,0,0>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a0_l0_e0_delta_nf0)

			);
}

//Ibar_G<e_q,0,0,0> = 0

//Ibar_G2<e_g,0,0,0> = 0

//Ibar_G2<e_q,0,0,0> = 0


//*****************************
//********   a = 1   ********
//*****************************

//**********************************************************
//********   l = 1   ****************   e = 1   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,1,1,1>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__g_a1_l1_e1_nf0)
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a1_l1_e1_delta_nf0 + Nf*(_const_Ibar_g__g_a1_l1_e1_delta_nf1 ))// != CuTe 1.1 but true
			+PD<+0>(fz,f1,p)*(_const_Ibar_g__g_a1_l1_e1_plus0_nf0)

			);
}

template<  >
inline double Ibar_G<e_q,1,1,1>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__q_a1_l1_e1_nf0)

			);
}

//Ibar_G2<e_g,1,1,1> = 0

//Ibar_G2<e_q,1,1,1> = 0


//**********************************************************
//********   l = 0   ****************   e = 2   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,1,0,2>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a1_l0_e2_delta_nf0)

			);
}

template<  >
inline double Ibar_G<e_q,1,0,2>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__q_a1_l0_e2_nf0)

			);
}

template<  >
inline double Ibar_G2<e_g,1,0,2>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g2_g_a1_l0_e2_nf0)

			);
}

template<  >
inline double Ibar_G2<e_q,1,0,2>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g2_q_a1_l0_e2_nf0)

			);
}


//*****************************
//********   a = 2   ********
//*****************************

//**********************************************************
//********   l = 2   ****************   e = 2   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,2,2,2>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__g_a2_l2_e2_nf0 + Nf*(p._func_Ibar_g__g_a2_l2_e2_nf1 ))
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a2_l2_e2_delta_nf0)
			+PD<+0>(fz,f1,p)*(_const_Ibar_g__g_a2_l2_e2_plus0_nf0 + Nf*(_const_Ibar_g__g_a2_l2_e2_plus0_nf1 ))
			+PD<+1>(fz,f1,p)*(_const_Ibar_g__g_a2_l2_e2_plus1_nf0)

			);
}

template<  >
inline double Ibar_G<e_q,2,2,2>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__q_a2_l2_e2_nf0)

			);
}

//Ibar_G2<e_g,2,2,2> = 0

//Ibar_G2<e_q,2,2,2> = 0


//**********************************************************
//********   l = 1   ****************   e = 3   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,2,1,3>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__g_a2_l1_e3_nf0 + Nf*(p._func_Ibar_g__g_a2_l1_e3_nf1 ))
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a2_l1_e3_delta_nf0 + Nf*(_const_Ibar_g__g_a2_l1_e3_delta_nf1 ))
			+PD<+0>(fz,f1,p)*(_const_Ibar_g__g_a2_l1_e3_plus0_nf0 + Nf*(_const_Ibar_g__g_a2_l1_e3_plus0_nf1 ))

			);
}

template<  >
inline double Ibar_G<e_q,2,1,3>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__q_a2_l1_e3_nf0 + Nf*(p._func_Ibar_g__q_a2_l1_e3_nf1 ))

			);
}

template<  >
inline double Ibar_G2<e_g,2,1,3>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g2_g_a2_l1_e3_nf0 + Nf*(p._func_Ibar_g2_g_a2_l1_e3_nf1 ))

			);
}

template<  >
inline double Ibar_G2<e_q,2,1,3>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g2_q_a2_l1_e3_nf0 + Nf*(p._func_Ibar_g2_q_a2_l1_e3_nf1 ))

			);
}


//**********************************************************
//********   l = 0   ****************   e = 4   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,2,0,4>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__g_a2_l0_e4_nf0 + Nf*(p._func_Ibar_g__g_a2_l0_e4_nf1 ))
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a2_l0_e4_delta_nf0 + Nf*(_const_Ibar_g__g_a2_l0_e4_delta_nf1 ))
			+PD<+0>(fz,f1,p)*(_const_Ibar_g__g_a2_l0_e4_plus0_nf0 + Nf*(_const_Ibar_g__g_a2_l0_e4_plus0_nf1 ))

			);
}

template<  >
inline double Ibar_G<e_q,2,0,4>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__q_a2_l0_e4_nf0 + Nf*(p._func_Ibar_g__q_a2_l0_e4_nf1 ))

			);
}

//Ibar_G2<e_g,2,0,4> = 0

//Ibar_G2<e_q,2,0,4> = 0


//*****************************
//********   a = 3   ********
//*****************************

//**********************************************************
//********   l = 3   ****************   e = 3   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,3,3,3>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__g_a3_l3_e3_nf0 + Nf*(p._func_Ibar_g__g_a3_l3_e3_nf1 + Nf*(p._func_Ibar_g__g_a3_l3_e3_nf2 ) ))
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a3_l3_e3_delta_nf0)
			+PD<+0>(fz,f1,p)*(_const_Ibar_g__g_a3_l3_e3_plus0_nf0 + Nf*(_const_Ibar_g__g_a3_l3_e3_plus0_nf1 + Nf*(_const_Ibar_g__g_a3_l3_e3_plus0_nf2 ) ))
			+PD<+1>(fz,f1,p)*(0.)
			+PD<+2>(fz,f1,p)*(_const_Ibar_g__g_a3_l3_e3_plus2_nf0)

			);
}

template<  >
inline double Ibar_G<e_q,3,3,3>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__q_a3_l3_e3_nf0 + Nf*(p._func_Ibar_g__q_a3_l3_e3_nf1 ))

			);
}

//Ibar_G2<e_g,3,3,3> = 0

//Ibar_G2<e_q,3,3,3> = 0


//**********************************************************
//********   l = 2   ****************   e = 4   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,3,2,4>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__g_a3_l2_e4_nf0 + Nf*(p._func_Ibar_g__g_a3_l2_e4_nf1 + Nf*(p._func_Ibar_g__g_a3_l2_e4_nf2 ) ))
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a3_l2_e4_delta_nf0 + Nf*(_const_Ibar_g__g_a3_l2_e4_delta_nf1 + Nf*(_const_Ibar_g__g_a3_l2_e4_delta_nf2 ) ))
			+PD<+0>(fz,f1,p)*(_const_Ibar_g__g_a3_l2_e4_plus0_nf0 + Nf*(_const_Ibar_g__g_a3_l2_e4_plus0_nf1 ))
			+PD<+1>(fz,f1,p)*(_const_Ibar_g__g_a3_l2_e4_plus1_nf0 + Nf*(_const_Ibar_g__g_a3_l2_e4_plus1_nf1 ))

			);
}

template<  >
inline double Ibar_G<e_q,3,2,4>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__q_a3_l2_e4_nf0 + Nf*(p._func_Ibar_g__q_a3_l2_e4_nf1 + Nf*(p._func_Ibar_g__q_a3_l2_e4_nf2 ) ))

			);
}

//Ibar_G2<e_g,3,2,4> = 0

//Ibar_G2<e_q,3,2,4> = 0


//**********************************************************
//********   l = 1   ****************   e = 5   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,3,1,5>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__g_a3_l1_e5_nf0 + Nf*(p._func_Ibar_g__g_a3_l1_e5_nf1 + Nf*(p._func_Ibar_g__g_a3_l1_e5_nf2 ) ))
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a3_l1_e5_delta_nf0 + Nf*(_const_Ibar_g__g_a3_l1_e5_delta_nf1 + Nf*(_const_Ibar_g__g_a3_l1_e5_delta_nf2 ) ))
			+PD<+0>(fz,f1,p)*(_const_Ibar_g__g_a3_l1_e5_plus0_nf0 + Nf*(_const_Ibar_g__g_a3_l1_e5_plus0_nf1 + Nf*(_const_Ibar_g__g_a3_l1_e5_plus0_nf2 ) ))
			+PD<+1>(fz,f1,p)*(_const_Ibar_g__g_a3_l1_e5_plus1_nf0 + Nf*(_const_Ibar_g__g_a3_l1_e5_plus1_nf1 ))

			);
}

template<  >
inline double Ibar_G<e_q,3,1,5>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__q_a3_l1_e5_nf0 + Nf*(p._func_Ibar_g__q_a3_l1_e5_nf1 + Nf*(p._func_Ibar_g__q_a3_l1_e5_nf2 ) ))

			);
}

//Ibar_G2<e_g,3,1,5> = 0

//Ibar_G2<e_q,3,1,5> = 0


//*****************************
//********   a = 4   ********
//*****************************

//**********************************************************
//********   l = 4   ****************   e = 4   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,4,4,4>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__g_a4_l4_e4_nf0 + Nf*(p._func_Ibar_g__g_a4_l4_e4_nf1 + Nf*(p._func_Ibar_g__g_a4_l4_e4_nf2 + Nf*(p._func_Ibar_g__g_a4_l4_e4_nf3 ) ) ))
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a4_l4_e4_delta_nf0 + Nf*(_const_Ibar_g__g_a4_l4_e4_delta_nf1 + Nf*(_const_Ibar_g__g_a4_l4_e4_delta_nf2 ) ))
			+PD<+0>(fz,f1,p)*(_const_Ibar_g__g_a4_l4_e4_plus0_nf0 + Nf*(_const_Ibar_g__g_a4_l4_e4_plus0_nf1 + Nf*(_const_Ibar_g__g_a4_l4_e4_plus0_nf2 + Nf*(_const_Ibar_g__g_a4_l4_e4_plus0_nf3 ) ) ))
			+PD<+1>(fz,f1,p)*(_const_Ibar_g__g_a4_l4_e4_plus1_nf0 + Nf*(_const_Ibar_g__g_a4_l4_e4_plus1_nf1 + Nf*(_const_Ibar_g__g_a4_l4_e4_plus1_nf2 ) ))
			+PD<+2>(fz,f1,p)*(_const_Ibar_g__g_a4_l4_e4_plus2_nf0 + Nf*(_const_Ibar_g__g_a4_l4_e4_plus2_nf1 ))
			+PD<+3>(fz,f1,p)*(_const_Ibar_g__g_a4_l4_e4_plus3_nf0)

			);
}

template<  >
inline double Ibar_G<e_q,4,4,4>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__q_a4_l4_e4_nf0 + Nf*(p._func_Ibar_g__q_a4_l4_e4_nf1 + Nf*(p._func_Ibar_g__q_a4_l4_e4_nf2 ) ))

			);
}

//Ibar_G2<e_g,4,4,4> = 0

//Ibar_G2<e_q,4,4,4> = 0


//**********************************************************
//********   l = 3   ****************   e = 5   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,4,3,5>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__g_a4_l3_e5_nf0 + Nf*(p._func_Ibar_g__g_a4_l3_e5_nf1 + Nf*(p._func_Ibar_g__g_a4_l3_e5_nf2 + Nf*(p._func_Ibar_g__g_a4_l3_e5_nf3 ) ) ))
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a4_l3_e5_delta_nf0 + Nf*(_const_Ibar_g__g_a4_l3_e5_delta_nf1 + Nf*(_const_Ibar_g__g_a4_l3_e5_delta_nf2 + Nf*(_const_Ibar_g__g_a4_l3_e5_delta_nf3 ) ) ))
			+PD<+0>(fz,f1,p)*(_const_Ibar_g__g_a4_l3_e5_plus0_nf0 + Nf*(_const_Ibar_g__g_a4_l3_e5_plus0_nf1 + Nf*(_const_Ibar_g__g_a4_l3_e5_plus0_nf2 ) ))
			+PD<+1>(fz,f1,p)*(_const_Ibar_g__g_a4_l3_e5_plus1_nf0 + Nf*(_const_Ibar_g__g_a4_l3_e5_plus1_nf1 + Nf*(_const_Ibar_g__g_a4_l3_e5_plus1_nf2 ) ))
			+PD<+2>(fz,f1,p)*(_const_Ibar_g__g_a4_l3_e5_plus2_nf0 + Nf*(_const_Ibar_g__g_a4_l3_e5_plus2_nf1 ))

			);
}

template<  >
inline double Ibar_G<e_q,4,3,5>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__q_a4_l3_e5_nf0 + Nf*(p._func_Ibar_g__q_a4_l3_e5_nf1 + Nf*(p._func_Ibar_g__q_a4_l3_e5_nf2 + Nf*(p._func_Ibar_g__q_a4_l3_e5_nf3 ) ) ))

			);
}

//Ibar_G2<e_g,4,3,5> = 0

//Ibar_G2<e_q,4,3,5> = 0


//*****************************
//********   a = 5   ********
//*****************************

//**********************************************************
//********   l = 5   ****************   e = 5   ********
//**********************************************************

template<  >
inline double Ibar_G<e_g,5,5,5>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__g_a5_l5_e5_nf0 + Nf*(p._func_Ibar_g__g_a5_l5_e5_nf1 + Nf*(p._func_Ibar_g__g_a5_l5_e5_nf2 + Nf*(p._func_Ibar_g__g_a5_l5_e5_nf3 + Nf*(p._func_Ibar_g__g_a5_l5_e5_nf4 ) ) ) ))
			+PD<-1>(fz,f1,p)*(_const_Ibar_g__g_a5_l5_e5_delta_nf0 + Nf*(_const_Ibar_g__g_a5_l5_e5_delta_nf1 + Nf*(_const_Ibar_g__g_a5_l5_e5_delta_nf2 + Nf*(_const_Ibar_g__g_a5_l5_e5_delta_nf3 ) ) ))
			+PD<+0>(fz,f1,p)*(_const_Ibar_g__g_a5_l5_e5_plus0_nf0 + Nf*(_const_Ibar_g__g_a5_l5_e5_plus0_nf1 + Nf*(_const_Ibar_g__g_a5_l5_e5_plus0_nf2 + Nf*(_const_Ibar_g__g_a5_l5_e5_plus0_nf3 + Nf*(_const_Ibar_g__g_a5_l5_e5_plus0_nf4 ) ) ) ))
			+PD<+1>(fz,f1,p)*(_const_Ibar_g__g_a5_l5_e5_plus1_nf0 + Nf*(_const_Ibar_g__g_a5_l5_e5_plus1_nf1 + Nf*(_const_Ibar_g__g_a5_l5_e5_plus1_nf2 + Nf*(_const_Ibar_g__g_a5_l5_e5_plus1_nf3 ) ) ))
			+PD<+2>(fz,f1,p)*(_const_Ibar_g__g_a5_l5_e5_plus2_nf0 + Nf*(_const_Ibar_g__g_a5_l5_e5_plus2_nf1 + Nf*(_const_Ibar_g__g_a5_l5_e5_plus2_nf2 ) ))
			+PD<+3>(fz,f1,p)*(_const_Ibar_g__g_a5_l5_e5_plus3_nf0 + Nf*(_const_Ibar_g__g_a5_l5_e5_plus3_nf1 ))
			+PD<+4>(fz,f1,p)*(_const_Ibar_g__g_a5_l5_e5_plus4_nf0)

			);
}

template<  >
inline double Ibar_G<e_q,5,5,5>(double fz, double f1, const Ibar_G_Parameter &p, int Nf){
	return (
			 fz*(p._func_Ibar_g__q_a5_l5_e5_nf0 + Nf*(p._func_Ibar_g__q_a5_l5_e5_nf1 + Nf*(p._func_Ibar_g__q_a5_l5_e5_nf2 + Nf*(p._func_Ibar_g__q_a5_l5_e5_nf3 ) ) ))

			);
}

//Ibar_G2<e_g,5,5,5> = 0

//Ibar_G2<e_q,5,5,5> = 0



} // namespace NS_Ibar
} // namespace CuTe

#endif // IBAR_G_FUNCTIONS_H


