#ifndef PlusDists_h
#define PlusDists_h

#include <vector>
#include <iostream>
#include <cstdlib>

namespace NS_PlusDists {

using std::cerr;
using std::endl;
using std::vector;
using std::string;
using std::stringstream;


static const int MAX_PLUS = 4;

class PD_Params {
protected:
	vector<double> _lbxOn;
	vector<double> _lbz;
public:
	double x;

	double bx;	//1-x
	double ibx;	//1/bx

	double lbx;	//log(1-x)


	double z;

	double z2;	//z^2
	double iz;	//1/z

	double bz;	//1-z
	double ibz;	//1/bz

	double zpp;	//1+z
	double izpp;//1/zpp

	double lbz;


	inline double Lbz(int n)const{
//		if(n>MAX_PLUS){	// debug
//			std::cerr << "MAX_PLUS in PD_Params"	<< endl;
//			exit(1);
//		}
		return _lbz[n];
	}

	inline double LbxOn(int n)const{
//		if(n>MAX_PLUS+1){	// debug
//			std::cerr << "MAX_PLUS in PD_Params"	<< endl;
//			exit(1);
//		}
		return _lbxOn[n];
	}

	inline virtual void update(double z){
//		cout	<<"\t->PD_Params::update("<<z<<")"	<<endl;
		this->z	= (z);

		z2	= z*z;
		iz	= 1/z;

		bz	= 1-z;
		ibz = 1/bz;

		zpp	= 1+z;
		izpp= 1/zpp;

		lbz	= log(bz);

		_lbz.resize(MAX_PLUS+1);
		double tmp = 1.;
		for(int i=1;i<=MAX_PLUS;++i){
			_lbz[i] = tmp*= lbz;
		}

//		cout	<<"\t<-PD_Params::update("<<z<<")"	<<endl;
	}

	inline virtual void set_x(double x){
//		cout	<<"PD_Params::set_x("<<x<<")"	<<endl;
		this->x	= x;

		bx	= (1-x);
		ibx	= 1/bx;
		lbx	= log(bx);

		_lbxOn.resize(MAX_PLUS+2);
		double tmp = _lbxOn[1]	= lbx;
		for(int i=2;i<=MAX_PLUS+1;++i){
			tmp*= lbx;
			_lbxOn[i] = tmp/static_cast<double>(i);
		}
	}

	inline PD_Params()
	:_lbxOn(MAX_PLUS+2,1.),_lbz(MAX_PLUS+1,1.)
	,x(0.),bx(0.),ibx(0.),lbx(0.)
	,z(0.),z2(0.),iz(0.),bz(0.),ibz(0.),zpp(0.),izpp(0.),lbz(0.){

	}

	inline virtual ~PD_Params(){

	}

	inline virtual const string to_String(int ntabs=0)const{
		stringstream ss;

		string tabs("");
		for(int i=1;i<=ntabs; ++i) tabs += string("\t");

		ss	<<tabs<<"PD_Params:"<<endl
			<<tabs<<"\t"<<"x="<<x<<endl
			<<tabs<<"\t\t"<<"bx="<<bx<<"\t"<<"ibx="<<ibx<<endl
			<<tabs<<"\t";
		for(int i=0; i<MAX_PLUS+2;++i)	ss<<"\t"<<"LbxOn("<<i<<")="<<LbxOn(i);
		ss	<<endl
			<<tabs<<"\t"<<"z="<<z<<endl
			<<tabs<<"\t\t"<<"z2="<<z2<<"\t"<<"iz="<<iz<<"\t"<<"bz="<<bz<<"\t"<<"ibz="<<ibz<<endl
			<<tabs<<"\t";
		for(int i=0; i<MAX_PLUS+1;++i)	ss<<"\t"<<"Lbz("<<i<<")="<<Lbz(i);
		ss	<<endl;

		return ss.str();
	}
};

template<int N>
inline double PD(double fz, double f1, const PD_Params& p){
	const double safe(f1*p.ibx*p.LbxOn(N+1));
	const double unsafe(p.Lbz(N)*(fz-f1)*p.ibz);
	if(safe!=safe){
		cerr	<<"NAN PD<1> safe"<<endl;
	}
	if(unsafe!=unsafe) return safe;
	return safe + unsafe;
}

template<>
inline double PD<-1>(double fz, double f1, const PD_Params& p){
	return f1*p.ibx;
}

template<>
inline double PD<0>(double fz, double f1, const PD_Params& p){
	const double safe(f1*p.ibx*p.lbx);
	const double unsafe((fz-f1)*p.ibz);
	if(safe!=safe){
		cerr	<<"NAN PD<1> safe"<<endl;
	}
	if(unsafe!=unsafe) return safe;
	return safe + unsafe;
}

template<>
inline double PD<1>(double fz, double f1, const PD_Params& p){
	const double safe(f1*p.ibx*p.LbxOn(2));
	const double unsafe(p.lbz*(fz-f1)*p.ibz);
	if(safe!=safe){
		cerr	<<"NAN PD<1> safe"<<endl;
	}
	if(unsafe!=unsafe){
//		cerr	<<"NAN PD<1> unsafe"<<endl;
		return safe;
	}
	return safe + unsafe;
}



}  // namespace NS_PlusDists

#endif
