#ifndef QDCS_HARDFUNC_H
#define QDCS_HARDFUNC_H


#include "param.h"
#include "polynom.h"

namespace CuTe{
namespace NS_Hard{

typedef NS_Polynom::Pol<double> Pol_a;

using NS_CS_Params::CS_Parameter;
using NS_CS_Params::CS_Parameter_LL;
using namespace NS_enums::NS_PROD;
using namespace NS_enums::NS_RES;
using NS_LHAFUNC::A_strong;

using std::complex;
using std::endl;

//Forward declaration of exponents
class C_exp;
class U_exp;

class _Hardfunc_QCD5:public Pol_a{
protected:
	const CS_Parameter *p;

	int order_a;
	double M_t;

	C_exp *C_t;
	U_exp *U_t;

	Pol_a	lnC;
	Pol_a	lnU;

	_Hardfunc_QCD5()
	:Pol_a(2,0.)
	,p(NULL)
	,order_a()
	,M_t()
	,C_t(NULL),U_t(NULL)
	,lnC(Pol_a(2,0.)),lnU(Pol_a(2,0.)){

	}

	void _init(const CS_Parameter *p, const PROD prod, const bool resummation, const int order_a, const double M, const double M_t, const A_strong *AS, const double Gamma_Cusp_3_Pade_Factor);

	virtual ~_Hardfunc_QCD5();

	_Hardfunc_QCD5& operator=(const _Hardfunc_QCD5 &H){
		set_equalt_to(H);
		lnC.set_equalt_to(H.lnC);
		lnU.set_equalt_to(H.lnU);
		return *this;
	}

	virtual void _change_AS(const A_strong* AS);
private:
	_Hardfunc_QCD5(const _Hardfunc_QCD5 &H);

public:
	void change_AS(const CS_Parameter* p);

};

class _Hardfunc_SCET:public _Hardfunc_QCD5{
protected:
	complex<double> M;

	C_exp *C_SV;
	U_exp *U_SV;

	_Hardfunc_SCET()
	:_Hardfunc_QCD5()
	,M()
	,C_SV(NULL),U_SV(NULL){

	}

	void _init(const CS_Parameter *p, const PROD prod, const bool resummation, const int order_a, const double M, const double M_t, const A_strong *AS, const double Gamma_Cusp_3_Pade_Factor);

	Pol_a& _update_lnC(const complex<double> &M, const complex<double> &mu_h, const double M_t, const double mu_t);
	Pol_a& _update_lnU(const complex<double> &M, const complex<double> &mu_h, const complex<double> &mu);
	Pol_a& _update_lnU(const complex<double> &M, const complex<double> &mu_h, const double M_t, const double mu_t, const complex<double> &mu);

	virtual ~_Hardfunc_SCET();
	virtual void _change_AS(const A_strong* AS);

private:
	_Hardfunc_SCET(const _Hardfunc_SCET &H);

public:

};

class Hardfunc_FO:public _Hardfunc_QCD5{
protected:
	Pol_a& _update_mu(const double mu, const double expfactor);
public:

	inline Pol_a& update_mu(const double mu, const double expfactor=1.){
		return _update_mu(mu,+expfactor);
	}
	inline Pol_a& update_mu_inverse(const double mu, const double expfactor=1.){
		return _update_mu(mu,-expfactor);
	}

	void init(const CS_Parameter *p);

	Hardfunc_FO()
	:_Hardfunc_QCD5(){

	}

	Hardfunc_FO& operator=(const Hardfunc_FO &H){
		p	= (H.p);
		if(p!=NULL)	init(p);
		_Hardfunc_QCD5::operator=(H);
		return *this;
	}

	Hardfunc_FO(const Hardfunc_FO &H)
	:_Hardfunc_QCD5(){
		operator=(H);
	}

};

class Hardfunc_SQT:public _Hardfunc_SCET{
protected:
	bool pi2;
public:

	Pol_a& update_mu(const double mu, const double expfactor=1.);
	inline Pol_a& update_mu_inverse(const double mu, const double expfactor=1.){
		return update_mu(mu,-expfactor);
	}
	void init(const CS_Parameter *p,const bool pi2=false);

	Hardfunc_SQT()
	:_Hardfunc_SCET()
	,pi2(){

	}

	Hardfunc_SQT& operator=(const Hardfunc_SQT &H){
		p	= (H.p);
		if(p!=NULL)	init(p,H.pi2);
		_Hardfunc_SCET::operator=(H);
		return *this;
	}

	Hardfunc_SQT(const Hardfunc_SQT &H)
	:_Hardfunc_SCET()
	,pi2(){
		operator=(H);
	}

};


class Hardfunc_LL:public _Hardfunc_SCET{
protected:
	complex<double> mu_h;
	double mu_t;

	bool exponent;
	bool pi2;

	void _init(const CS_Parameter *p, const int order_a, const bool exponent, const bool pi2);

public:

	Pol_a& update_mu(const double mu, const double expfactor=1.);
	virtual void init(const CS_Parameter *p);

	Hardfunc_LL()
	:_Hardfunc_SCET()
	,mu_h(),mu_t()
	,exponent(),pi2(){

	}

	Hardfunc_LL& operator=(const Hardfunc_LL &H){
		p	= (H.p);
		if(p!=NULL)	init(p);
		_Hardfunc_SCET::operator=(H);
		return *this;
	}

	Hardfunc_LL(const Hardfunc_LL &H)
	:_Hardfunc_SCET()
	,mu_h(),mu_t()
	,exponent(),pi2(){
		operator=(H);
	}

};

class MS_Factor: public Pol_a {
protected:
	const CS_Parameter_LL *p;
	static const double MU_LIMIT;// lower limit for mu_ms = qT; linear interpolation to R_ms(0)=0 below

	int order_a;
	complex<double> M;		//= i*M	=> M^2 -> -M^2
	complex<double> mu_h;	//= (pi2 ? i*mu_h : mu_h)

	C_exp *C_SV;
	U_exp *U_SV;

	Pol_a	lnC_RES;
	Pol_a	lnC_SQT;
	Pol_a	lnU_RES;
//	Pol_a	lnU_SQT;

	NS_enums::NS_MS::MS ms;

	bool pi2;
	bool exponentiate_U;
	bool as_number;
	bool order_mm;


	void _update_mu(const double mu, const double expfactor);

public:

	void init(const CS_Parameter_LL *p, const NS_enums::NS_MS::MS ms, const bool as_number=true, const bool exponentiate_U=true, const bool order_mm=true);

	Pol_a& update_mu(const double qT, double mu, const double mu_star, const double expfactor=1.);

	MS_Factor()
	:Pol_a(2,0.)
	,p(NULL)
	,order_a()
	,M(),mu_h()
	,C_SV(NULL),U_SV(NULL)
	,lnC_RES(2,0.),lnC_SQT(2,0.),lnU_RES(2,0.)//,lnU_SQT(2,0.)
	,ms()
	,pi2()
	,exponentiate_U()
	,as_number()
	,order_mm(){

	}

	MS_Factor(const MS_Factor &F)
	:Pol_a(2,0.)
	,p(NULL)
	,order_a()
	,M(),mu_h()
	,C_SV(NULL),U_SV(NULL)
	,lnC_RES(2,0.),lnC_SQT(2,0.),lnU_RES(2,0.)//,lnU_SQT(2,0.)
	,ms()
	,exponentiate_U()
	,as_number()
	,order_mm(){
		operator=(F);
	}

	MS_Factor& operator=(const MS_Factor &F){
		p	= (F.p);
		if(p!=NULL)	init(p,F.ms,F.as_number,F.exponentiate_U,F.order_mm);
		set_equalt_to(F);
		lnC_RES.set_equalt_to(F.lnC_RES);
		lnC_SQT.set_equalt_to(F.lnC_SQT);
		lnU_RES.set_equalt_to(F.lnU_RES);
//		lnU_SQT.set_equalt_to(F.lnU_SQT);
		return *this;
	}

	std::ostream& to_stream(std::ostream &os, const int ntabs=0);

	~MS_Factor();

	void change_AS(const CS_Parameter *p);

};

class MS_Factor_test{
protected:
	const CS_Parameter *p;

	int order_a;
	complex<double> M;

	double mu_h;

	C_exp *C_SV;
	U_exp *U_SV;

	std::vector<double> results;

	Pol_a& ln_C(Pol_a & P, const complex<double> &mu_h)const;
	Pol_a& ln_U(Pol_a & P, const complex<double> &mu_h, const complex<double> &mu)const;

public:

	void init(const CS_Parameter *p, const int order_a);
	void init(const CS_Parameter *p);

	const std::vector<double>& update(const double mu);

	std::ostream& FL(std::ostream &os, const std::string &suffix="")const;
	std::ostream& result(std::ostream &os)const{
		for(uint i=0; i<results.size(); ++i) os<<"\t"<<results[i];
		return os;
	}

	MS_Factor_test()
	:p(NULL)
	,order_a(0)
	,M(),mu_h()
	,C_SV(NULL),U_SV(NULL){

	}

	~MS_Factor_test();


};



}  // namespace NS_Hard
}  // namespace CuTe

#endif
