#ifndef QDCS_SIGMA0_H
#define QDCS_SIGMA0_H

#include "enums.h"

namespace CuTe {

namespace NS_CS_Params {
	class CS_Parameter;
}  // namespace NS_CS_Params


namespace NS_Kinematics {
	class Kinematics;
}  // namespace NS_Kinematics


namespace NS_QDCS {

class _Sigma_0;

using NS_Kinematics::Kinematics;
using namespace NS_enums::NS_PROD;
using NS_CS_Params::CS_Parameter;

class Sigma_0{
protected:
	const CS_Parameter *p;

	PROD prod;
	const double *as;
	_Sigma_0* sigma0;

	double S0;

private:
	void _exit_s0();
	void _init_s0();

public:

	void init(const CS_Parameter* p);

	inline Sigma_0()
	:p(NULL)
	,prod(PROD_WRONG_STATEMENT)
	,as(NULL),sigma0(NULL)
	,S0(){
//		DOUT dout("Sigma_0");
	}

	inline Sigma_0(const CS_Parameter *p)
	:p(NULL)
	,prod(PROD_WRONG_STATEMENT)
	,as(NULL),sigma0(NULL)
	,S0(){
		init(p);
	}

	inline Sigma_0(const Sigma_0& S)
	:p(NULL)
	,prod(PROD_WRONG_STATEMENT)
	,as(NULL),sigma0(NULL)
	,S0(S.S0){
//		DOUT dout("Sigma_0","S");
		if(p!=NULL)	init(p);
	}

	inline Sigma_0& operator= (const Sigma_0& S){
//		DOUT dout("Sigma_0::operator=","S");
		_exit_s0();

		if(S.p!=NULL)	init(S.p);

		S0		= (S.S0);

		return *this;
	}

	inline ~Sigma_0(){
		_exit_s0();
	}

	double update_qT_mu(const Kinematics *Kin,const CS_Parameter* p);
	double update_qT_mu(const double qT,const CS_Parameter* p);

	inline double operator() ()const{ return S0; }
	inline double operator() (const Kinematics *Kin,const CS_Parameter* p){ return update_qT_mu(Kin,p); }
};

}  // namespace NS_CROSSSECTION



}  // namespace CuTe

#endif



