#ifndef TDPDF_INTEGRAND_H
#define TDPDF_INTEGRAND_H

#include "Res_K_Integral.h"
#include "Ibar.h"

namespace CuTe {

namespace NS_TDPDF {

using namespace NS_enums::NS_TD_PARTON;


using std::cerr;
using std::complex;
using std::endl;
using std::string;
using std::vector;

namespace NS_TDPDF_Integrand{

enum E_INT {
	INT_B
	,INT_Btilde
	,INT_XI_B
	,INT_XI_Btilde
	,INT_B_over_Phi
	,INT_Btilde_over_Phi
};

using namespace NS_enums::NS_NP;
using namespace NS_enums::NS_BFUNCS;
using namespace NS_enums::NS_PROD;
using namespace NS_enums::NS_RES;
using namespace NS_enums::NS_PC;

using NS_Polynom::Pol;
using NS_Ibar::Pol_a_l;

using NS_Ibar::VectorbarI_DY;
using NS_Ibar::VectorbarI_H;
using NS_Ibar::Ibar_G_Parameter;
using NS_Ibar::Ibar_Q_Parameter;

using NS_Resumed::_exp_pol_;
using NS_Resumed::H_exp_Pol;
using NS_Resumed::_K_Integral_;
using NS_Resumed::K_LLO;

using NS_Utils::NS_pow::pow_2;



class Half_G_Exp:public NS_Resumed::G_exp_Pol {
public:
	inline void _set_mu(){
//		NS_Utils::DOUT dout("Half_G_Exp::_set_mu");
		G_exp_Pol::_set_mu();

		mul_this_by_scalar(0.5);
	}

	inline void _set_mu(Pol< complex<double> > &Kexp,const complex<double> kexpfactor=1.){
//		NS_Utils::DOUT dout("Half_G_Exp::_set_mu","Kexp,kexpfactor");
		G_exp_Pol::_set_mu(Kexp,kexpfactor*0.5);

		mul_this_by_scalar(0.5);
	}

	Half_G_Exp():G_exp_Pol(){

	}

	Half_G_Exp(
			int order, POWER_COUNTING pc, bool exponent
			,const int *nf, const double *as, const double *LM
			,PROD prod
			, const double Gamma_Cusp_3_Pade_Factor
			, const double Anomaly_Exponent_F_3_0_Factor
			)
	:G_exp_Pol(){
		init(order,pc,exponent,nf,as,LM,prod,Gamma_Cusp_3_Pade_Factor,Anomaly_Exponent_F_3_0_Factor);
	}

};

class One_Exp:public _exp_pol_ {
public:
	double oned;
	int onei;
public:
	inline void _set_mu(){
//		NS_Utils::DOUT dout("One_Exp::_set_mu");
		order(0);
		_P[0].order(0);
		_P[0][0]=0.;
	}

	inline void _set_mu(Pol< complex<double> > &Kexp,const complex<double> kexpfactor=1.){
//		NS_Utils::DOUT dout("One_Exp::_set_mu","Kexp,kexpfactor");
		_set_mu();
	}

	One_Exp()
	:_exp_pol_(0,0)
	,oned(1.),onei(1){
		_init(0,PC_STD,false,&oned,&onei,PROD_Higgs);
	}

};

class K_LL_half_cut :public NS_Resumed::K_LL{
public:
//
//	template<CUTOFF cut>
//	inline static double cut_function(double xT, double LT,double LM, double Lambda){
//		NS_Utils::DOUT dout("K_LL_half_cut::cut_function");
//		return sqrt(K_LL::cut_function<cut>(xT,LT,LM,Lambda));
//	}

	inline virtual void _choose_integrand(){
		if((*Lambda)==0.){
			gsl_int.function = _integrand_wick_;
		}else{
			switch (cutoff) {
			case CUTOFF_ENHANCED:
				gsl_int.function = _integrand_cut_<CUTOFF_ENHANCED_HALF>;
				break;
			case CUTOFF_DIPOL:
				gsl_int.function = _integrand_cut_<CUTOFF_DIPOL_HALF>;
				break;
			case CUTOFF_GAUSS:
				gsl_int.function = _integrand_cut_<CUTOFF_DIPOL_HALF>;
				break;
			default:
				std::cerr	<<"WRONG CUTOFF -> No Cutoff"<<std::endl;
				gsl_int.function = _integrand_wick_;
				break;
			}
		}
	}

};

typedef double (*_Cutfunc_)(double xT,double Lambda,double LT, double LM);

class _TDD_PDF_Integrand{
public:
	typedef Pol<double>					Pol_l;

	typedef Pol< Pol_l >				Pol_ae_l;
	typedef vector< Pol_ae_l >			V_Pol_ae_l;
	typedef V_Pol_ae_l::iterator		IT_Pol_ae_l;

	typedef Pol_ae_l*					Pol_ae_l_ptr;
	typedef vector< Pol_ae_l_ptr >		V_Pol_ae_l_ptr;
	typedef V_Pol_ae_l_ptr::iterator	IT_Pol_ae_l_ptr;

	static const int NB = 15;

protected:
	V_Pol_ae_l _B;
	IT_Pol_ae_l _B_it;
	const IT_Pol_ae_l _B_beg,_B_end,_BG,_BG1,_BG2;

	V_Pol_ae_l_ptr _Bbar;
	IT_Pol_ae_l_ptr _Bbar_it;
	const IT_Pol_ae_l_ptr _Bbar_beg,_Bbar_end,_BbarG,_BbarG1,_BbarG2;

	vector<double> _norms;
	vector<double>::iterator _norm_it;
	const vector<double>::iterator _norm_beg;

	VectorbarI_DY I_q;
	VectorbarI_H  I_g;

	ENUM_B_FUNCS B;


	Vector<NS_enums::NS_Partons::N_FMAX+1,bool> active_flavours;

public:
	_exp_pol_ *h_S;
	_exp_pol_ *h_V;

	int order;
//	bool improved;
	POWER_COUNTING pc;
	bool exponent;

	const double *xT;
	const double *mu;

	const int 	 *nf;	// nf(mu)
	const double *as;	// as(mu)
	const double *LT;	// log( xT^2 mu^2 b_0^(-2) )


	CUTOFF cut_off;
	const double *Lambda;
	const double *LM;	//	= eta/(a*G0) = Log(M^2/mu^2)
	_Cutfunc_ 	_cut_func;
	double 		cut_factor;


	double Gamma_Cusp_3_Pade_Factor;
	double Anomaly_Exponent_F_3_0_Factor;

	const bool improved_pc()const{ return (pc==PC_IMP); }
protected:

	inline void _updated_Bbar(const Ibar_G_Parameter &IGP,const Ibar_Q_Parameter &IQP){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::_updated_Bbar","IGP,IQP");
		I_q.update_z(&IQP);
		I_g.update_z(&IGP);

//		cout	<<I_q.to_String(dout.get_ntab());
//		cout	<<I_g.to_String(dout.get_ntab());
	}

	inline void _B_to_Bbar(Pol_ae_l &B, const Pol_ae_l &Bbar){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::_B_to_Bbar","B,Bbar");
//		dout()	<<"Bbar(1.) = "<<Bbar<<endl;
		B.set_equalt_to(Bbar);
		B.set_of(*as);

//		dout()	<<"Bbar("<<*as<<") = "<<B<<endl;
		if(improved_pc()){
//			NS_Utils::DOUT dout("_TDD_PDF_Intgerand::_B_to_Bbar","improved");
			improve_Pol_a_l(B,order);
//			dout()	<<"imp Bbar("<<*as<<") = "<<B<<endl;
		}
	}

	inline void _B_to_B(Pol_ae_l &B, const Pol_ae_l &Bbar, Pol_ae_l& P){
		_B_to_Bbar(B,Bbar);
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::_B_to_B","B,Bbar,P");
//		dout()	<<"P="<<P<<endl;
		B.mul_this_by_Pol(P,order);
	}

	inline double _sum_up(const Pol_ae_l &B, double LT, double norm)const{
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::_sum_up","B,LT");
		int N(B.order());
		double _sum_up(B[0].of(LT));

		for(int n=1; n<=N; ++n){
			_sum_up += (B[n].of(LT));
		}


//		dout()	<<"N = "<<N<<endl;
//		dout()	<<"return B("<<*as<<",LT) = "<<B<<endl;
//		dout()	<<"return B("<<*as<<","<<LT<<") = "<<_sum_up<<endl;
		return _sum_up*norm;
	}

	inline void _update_B(const Ibar_G_Parameter &IGP,const Ibar_Q_Parameter &IQP){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::_update","IGP,IQP");
		_updated_Bbar(IGP,IQP);

		_B_to_B(*(_BG1),*(*(_BbarG1)),*h_S);
		_B_to_B(*(_BG2),*(*(_BbarG2)),*h_S);

//		(*_BG).set_equalt_to(*_BG1,order);
//		(*_BG).add_this_Pol(*_BG2,order);

		_B_it		= _B_beg;
		_Bbar_it	= _Bbar_beg;
		while(_B_it!=_BG1){
			if( (_B_it!=_BG) ){
				_B_to_B(*(_B_it++),*(*(_Bbar_it++)),*h_V);
			}else{
				++_B_it;
				++_Bbar_it;
			}
		}
	}

	inline void _update_B_i(ENUM_TDPDF_Parton i, const Ibar_G_Parameter &IGP,const Ibar_Q_Parameter &IQP){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::_update_B_i","i,IGP,IQP");
		_updated_Bbar(IGP,IQP);
		_B_it	= _B_beg + i;
		_Bbar_it= _Bbar_beg + i;


		if(i==e_B_G){
			_B_to_B(*(_BG1),*(*(_BbarG1)),*h_S);
			_B_to_B(*(_BG2),*(*(_BbarG2)),*h_S);
		}else if(i==e_B_G1){
			_B_to_B(*(_BG1),*(*(_BbarG1)),*h_S);
		}else if(i==e_B_G2){
			_B_to_B(*(_BG2),*(*(_BbarG2)),*h_S);
		}else{
			_B_to_B(*(_B_it),*(*(_Bbar_it)),*h_V);
		}
//		dout()<<"Bbar["<<i<<"]:"<<endl;
//		dout()<<*(*(_Bbar_it))<<endl;
//		dout()<<"B["<<i<<"]:"<<endl;
//		dout()<<*(_B_it)<<endl;
	}

	template< bool normalized >
	inline double* _fill_from_B(double ff[NB], double Jac=1.){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::_fill_from_B","ff,Jac");
		_B_it	= _B_beg;

		Jac	*= cut_factor;

		double *fG;

		if(normalized){
			_norm_it= _norm_beg;
			while(_B_it != _B_end){
				if(_B_it==_BG){
					fG	= (ff++);
					++_B_it;
					++_norm_it;
				}else{
					*(ff++)	= _sum_up(*(_B_it++),*LT,Jac*(*(_norm_it++)));
				}
			}
		}else{
			while(_B_it != _B_end){
				if(_B_it==_BG){
					fG	= (ff++);
					++_B_it;
				}else{
					*(ff++)	= _sum_up(*(_B_it++),*LT,Jac);
				}
			}
		}
		*fG	= sqrt(pow_2(ff[-1])+pow_2(ff[-2]));

		return ff;
	}

	template< bool normalized >
	inline double* _fill_from_B_i(ENUM_TDPDF_Parton i, double ff[NB], double Jac=1.){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::_fill_from_B_i","i,ff,Jac");
		if(i==e_B_G){
			_B_it	= _BG1;
		}else{
			_B_it	= _B_beg+i;
		}

		Jac	*= cut_factor;
		if(normalized) Jac*=_norms[i];

		int N((*_B_it).order());
		int n=0;
		for(; n<=N; ++n){
			int L((*_B_it)[n].order());
			int l=0;
			for(; l<=L; ++l){
				*(ff++)	= (*_B_it)[n][l]*Jac;
			}
			for(; l<=n; ++l){
				*(ff++)	= 0;
			}
		}
		for(; n<=order; ++n){
			for(int l=0; l<=n; ++l)	*(ff++)	= 0;
		}

//		dout()<<"B["<<i<<"]:"<<endl;
//		dout()<<(*(_B_it)).mul_by_scalar(Jac)<<endl;

		if(i==e_B_G){
			_B_it	= _BG2;
			int N((*_B_it).order());
			int n=0;
			for(; n<=N; ++n){
				int L((*_B_it)[n].order());
				int l=0;
				for(; l<=L; ++l){
					*(ff)	= sqrt(pow_2(*ff) +  pow_2((*_B_it)[n][l]*Jac));
					++ff;
				}
			}
		}

		return ff;
	}

	inline void _init_h(){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::_init_h","ff,Jac");
		delete h_S;	// No = NULL
		delete h_V;	// No = NULL

		switch (B) {
		case E_BFUNC_B:
			h_S	= new H_exp_Pol(order,pc,exponent,nf,as,PROD_Higgs,Gamma_Cusp_3_Pade_Factor);
			h_V	= new H_exp_Pol(order,pc,exponent,nf,as,PROD_Z,Gamma_Cusp_3_Pade_Factor);
			break;
		case E_BFUNC_FB:
			h_S	= new Half_G_Exp(order,pc,exponent,nf,as,LM,PROD_Higgs,Gamma_Cusp_3_Pade_Factor,Anomaly_Exponent_F_3_0_Factor);
			h_V	= new Half_G_Exp(order,pc,exponent,nf,as,LM,PROD_Z,Gamma_Cusp_3_Pade_Factor,Anomaly_Exponent_F_3_0_Factor);
			break;
		default:
			h_S	= new One_Exp;
			h_V	= new One_Exp;
			break;
		}

	}

	inline void _init_Bbar(){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::_init_Bbar");
		_Bbar_it	= _Bbar_beg;

		int i=0;
		for(	; i<6; ++i){	// 0-5 :: t_bar - d_bar
			*(_Bbar_it++) = &(I_q[i]);
		}

		*(_Bbar_it++) = NULL;	// G = G1 + G2
		++i;

		for(	; i<13; ++i){	// 7-12 :: d - t
			*(_Bbar_it++) = &(I_q[i]);
		}

		*(_Bbar_it++)	= &(I_g[0]);	// 13 :: G1
		++i;
		*(_Bbar_it++) 	= &(I_g[1]);	// 13 :: G2
		++i;

		if( (i!=NB)||(_Bbar_it!=_Bbar_end) ){
			cerr	<<"****** _TDD_PDF_Integrand::_init_Bbar() ******"<<endl;
			exit(1);
		}

	}

	inline void _set_norms(){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::_set_norms");
		for(int i=0; i<NB; ++i){
			if(i==6){
				_norms[i]	= I_g.get_B_LO_by_index(i);
			}else if(i>12){
				_norms[i]	= I_g.get_B_LO_by_index(i);
			}else{
				_norms[i]	= I_q.get_B_LO_by_index(i);
			}
			if(_norms[i]!=0.)	_norms[i] = 1./_norms[i];
		}
	}

public:

	inline void init(
			ENUM_B_FUNCS B
			,NS_LHAFUNC::VectorPDF &pdf
			, const double *xT, const double *mu, const int *nf, const double *as, const double *LT
			,int order, POWER_COUNTING pc, bool exponent, CUTOFF cut_off
			,const double *Lambda, const double *LM
			,const double Gamma_Cusp_3_Pade_Factor
			,const double Anomaly_Exponent_F_3_0_Factor
			){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::init");

		this->B  = B;

		this->order			= order;
//		this->improved		= improved;
		this->pc			= pc;
		this->exponent		= exponent;
		this->Gamma_Cusp_3_Pade_Factor	= Gamma_Cusp_3_Pade_Factor;
		this->Anomaly_Exponent_F_3_0_Factor	= Anomaly_Exponent_F_3_0_Factor;

		this->xT	= xT;
		this->mu	= mu;

		this->nf	= nf;
		this->as	= as;
		this->LT	= LT;

		this->cut_off	= cut_off;
		this->Lambda	= Lambda;
		this->LM		= LM;

		I_q.init(pdf,mu,nf,active_flavours.begin()/*, order, improved*/);
		I_g.init(pdf,mu,nf,active_flavours.begin()/*, order, improved*/);

		_init_h();

		switch (cut_off) {
		case CUTOFF_DIPOL:
			_cut_func	= K_LL_half_cut::cut_function<CUTOFF_DIPOL_HALF>;
			break;
		case CUTOFF_GAUSS:
			_cut_func	= K_LL_half_cut::cut_function<CUTOFF_GAUSS_HALF>;
			break;
		case CUTOFF_ENHANCED:
			_cut_func	= K_LL_half_cut::cut_function<CUTOFF_ENHANCED_HALF>;
			break;
		default:
			_cut_func	= K_LL_half_cut::cut_function<CUTOFF_NO>;
			break;
		}

	}



	inline void set_xi_xT_mu(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ){
//		NS_Utils::DOUT	dout("_TDD_PDF_Integrand::set_xi_xT_mu","IG,IQ");
//
//		dout()	<<"xT="<<*xT<<"\t"<<"Lambda="<<*Lambda<<"\t"<<"LT="<<*LT<<"\t"<<"LM="<<*LM<<endl;

		cut_factor	= _cut_func(*xT,*LT,*Lambda,*LM);
//		dout()	<<"cut_factor = "<<cut_factor<<endl;

		h_S->update_mu_exp();
		h_V->update_mu_exp();

//		dout()	<<"h_S.exp()  = "<<*h_S<<endl;
//		dout()	<<"h_V.exp()  = "<<*h_V<<endl;

		I_g.set_xi(&IG);
		I_q.set_xi(&IQ);

		_set_norms();
	}


	_TDD_PDF_Integrand()
	:I_q(),I_g()
	,h_S(NULL),h_V(NULL)
	,_B(NB,Pol_ae_l(7,Pol_l(7,0.)))
	,_B_it(),_B_beg(_B.begin()),_B_end(_B.end()),_BG(_B_beg+6),_BG1(_B_end-2),_BG2(_B_end-1)
	,_Bbar(NB)
	,_Bbar_it(),_Bbar_beg(_Bbar.begin()),_Bbar_end(_Bbar.end()),_BbarG(_Bbar_beg+6),_BbarG1(_Bbar_end-2),_BbarG2(_Bbar_end-1)
	,_norms(NB,0.)
	,_norm_it(),_norm_beg(_norms.begin())
	,B()
	,order(),pc(),exponent()
	,Gamma_Cusp_3_Pade_Factor(1.),Anomaly_Exponent_F_3_0_Factor(0.)
	,xT(NULL),mu(NULL)
	,nf(NULL),as(NULL),LT(NULL)
	,cut_off()
	,Lambda(NULL),LM(NULL)
	,_cut_func(NULL)
	,active_flavours(true){
		_init_Bbar();
	}


	_TDD_PDF_Integrand(const _TDD_PDF_Integrand &O)
	:I_q(O.I_q),I_g(O.I_g)
	,h_S(NULL),h_V(NULL)
	,_B(NB,Pol_ae_l(7,Pol_l(7,0.)))
	,_B_it(),_B_beg(_B.begin()),_B_end(_B.end()),_BG(_B_beg+6),_BG1(_B_end-2),_BG2(_B_end-1)
	,_Bbar(NB)
	,_Bbar_it(),_Bbar_beg(_Bbar.begin()),_Bbar_end(_Bbar.end()),_BbarG(_Bbar_beg+6),_BbarG1(_Bbar_end-2),_BbarG2(_Bbar_end-1)
	,_norms(NB,0.)
	,_norm_it(),_norm_beg(_norms.begin())
	,B(O.B)
	,order(O.order),pc(O.pc),exponent(O.exponent)
	,Gamma_Cusp_3_Pade_Factor(O.Gamma_Cusp_3_Pade_Factor)
	,Anomaly_Exponent_F_3_0_Factor(O.Anomaly_Exponent_F_3_0_Factor)
	,xT(O.xT),mu(O.mu)
	,nf(O.nf),as(O.as),LT(O.LT)
	,cut_off(O.cut_off)
	,Lambda(O.Lambda),LM(O.LM)
	,_cut_func(O._cut_func)
	,active_flavours(true){
		_init_Bbar();
		if( (O.h_S!=NULL)&&(O.h_V!=NULL) )_init_h();
	}


	_TDD_PDF_Integrand& operator=(const _TDD_PDF_Integrand &O){
		I_q			= (O.I_q);
		I_g			= (O.I_g);
		B			= (O.B);
		order		= (O.order);
		pc			= (O.pc);
		exponent	= (O.exponent);
		Gamma_Cusp_3_Pade_Factor		= (O.Gamma_Cusp_3_Pade_Factor);
		Anomaly_Exponent_F_3_0_Factor	= (O.Anomaly_Exponent_F_3_0_Factor);
		xT			= (O.xT);
		mu			= (O.mu);
		nf			= (O.nf);
		as			= (O.as);
		LT			= (O.LT);
		cut_off		= (O.cut_off);
		Lambda		= (O.Lambda);
		LM			= (O.LM);
		_cut_func	= (O._cut_func);

		_init_Bbar();

		if( (O.h_S!=NULL)&&(O.h_V!=NULL) ){
			_init_h();
		}else{
			delete h_S;	h_S = NULL;
			delete h_V;	h_V = NULL;
		}

		return *this;
	}

	~_TDD_PDF_Integrand(){
		delete h_S;	// No = NULL
		delete h_V;	// No = NULL
	}


	inline double* integrand_B(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac=1.){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::integrand_B");
		_update_B(IG,IQ);
		return _fill_from_B<false>(ff,Jac);
	}

	inline double* integrand_B_Pol_i(ENUM_TDPDF_Parton i, const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac=1.){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::integrand_B_Pol_i");
		_update_B_i(i,IG,IQ);
		return _fill_from_B_i<false>(i,ff,Jac);
	}

	inline double* integrand_B_over_phi(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac=1.){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::integrand_B_over_phi");
		_update_B(IG,IQ);
		return _fill_from_B<true>(ff,Jac);
	}

	inline double* integrand_B_over_phi_Pol_i(ENUM_TDPDF_Parton i, const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac=1.){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::integrand_B_over_phi_Pol_i");
		_update_B_i(i,IG,IQ);
		return _fill_from_B_i<true>(i,ff,Jac);
	}

	inline double* integrand_B_times_xi(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac=1.){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::integrand_B_times_xi");
		return integrand_B(IG,IQ,ff,Jac*(IQ.x));
	}

	inline double* integrand_B_times_xi_Pol_i(ENUM_TDPDF_Parton i, const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac=1.){
//		NS_Utils::DOUT dout("_TDD_PDF_Integrand::integrand_B_times_xi_Pol_i");
		return integrand_B_Pol_i(i,IG,IQ,ff,Jac*(IQ.x));
	}
};

class _TMD_PDF_Integrand:public _TDD_PDF_Integrand{
public:
	typedef Pol<double>			Pol_ae;
	typedef vector< Pol_ae >	V_Pol_ae;
	typedef V_Pol_ae::iterator	IT_Pol_ae;

protected:
	V_Pol_ae _BTilde;
	IT_Pol_ae _BTilde_it;
	const IT_Pol_ae _BTilde_beg,_BTilde_end,_BTildeG,_BTildeG1,_BTildeG2;

	K_LL_half_cut _K_LL_V,_K_LL_S;
	K_LLO _K_LLO_V,_K_LLO_S;

	const double *qT,*Lmu,*q_star,*acc;	//log(mu^2/qT^2)
	RES res;

	double etaB_V,etaB_S;	//= C_B * Gamma_0*a*log(M^2/mu^2)
	_K_Integral_ *K_V,*K_S;


//	inline void _updated_Bbar(const Ibar_G_Parameter &IGP,const Ibar_Q_Parameter &IQP)
//	inline void _B_to_Bbar(Pol_ae_l &B, const Pol_ae_l &Bbar)

//	inline void _B_to_B(Pol_ae_l &B, const Pol_ae_l &Bbar, Pol_ae_l& P)
	inline void _B_to_BTilde(Pol_ae &BTilde, Pol_ae_l &B, const Pol_ae_l &Bbar, _K_Integral_ *K){
		_B_to_B(B,Bbar,*(K->gexp));
		BTilde.set_equalt_to(K->replaceL(B));
	}

//	inline double _sum_up(const Pol_ae_l &B, double LT)const
	inline double _sum_up(const Pol_ae &BTilde)const{
		return BTilde.of_1(order);
	}

//	inline void _update_B(const Ibar_G_Parameter &IGP,const Ibar_Q_Parameter &IQP){
////		NS_Utils::DOUT dout("_TDD_PDF_Intgerand::_update","IGP,IQP");
//		_updated_Bbar(IGP,IQP);
//
//		_B_to_B(*(_BG1),*(*(_BbarG1)),*h_S);
//		_B_to_B(*(_BG2),*(*(_BbarG2)),*h_S);
//
//		(*_BG).set_equalt_to(*_BG1,order);
//		(*_BG).add_this_Pol(*_BG2,order);
//
//		_B_it		= _B_beg;
//		_Bbar_it	= _Bbar_beg;
//		while(_B_it!=_BG1){
//			if( (_B_it!=_BG) ){
//				_B_to_B(*(_B_it++),*(*(_Bbar_it++)),*h_V);
//			}else{
//				++_B_it;
//				++_Bbar_it;
//			}
//		}
//	}
	inline void _update_BTilde(const Ibar_G_Parameter &IGP,const Ibar_Q_Parameter &IQP){
		_updated_Bbar(IGP,IQP);


		_B_to_BTilde(*_BTildeG1,*_BG1,*(*(_BbarG1)),K_S);
		_B_to_BTilde(*_BTildeG2,*_BG2,*(*(_BbarG2)),K_S);


//		(*_BTildeG).set_equalt_to(*_BTildeG1,order);
//		(*_BTildeG).add_this_Pol(*_BTildeG2,order);

		_BTilde_it	= _BTilde_beg;
		_B_it		= _B_beg;
		_Bbar_it	= _Bbar_beg;
		while(_B_it!=_BG1){
			if( (_B_it!=_BG) ){
				_B_to_BTilde(*(_BTilde_it++),*(_B_it++),*(*(_Bbar_it++)),K_V);
			}else{
				++_BTilde_it;
				++_B_it;
				++_Bbar_it;
			}
		}
	}

//	inline double* _fill_from_B(double ff[NB], double Jac=1.)
	template< bool normalized >
	inline double* _fill_from_BTilde(double ff[NB], double Jac=1.){
		_BTilde_it	= _BTilde_beg;

		double *fG;

		if(normalized){
			_norm_it= _norm_beg;
			while(_BTilde_it != _BTilde_end){
				if(_BTilde_it==_BTildeG){
					fG	= (ff++);
					++_BTilde_it;
					++_norm_it;
				}else{
					*(ff++)	= Jac*(*(_norm_it++))*_sum_up(*(_BTilde_it++));
				}
			}
		}else{
			while(_BTilde_it != _BTilde_end){
				if(_BTilde_it==_BTildeG){
					fG	= (ff++);
					++_BTilde_it;
				}else{
					*(ff++)	= Jac*_sum_up(*(_BTilde_it++));
				}
			}
		}

		*fG	= sqrt(pow_2(ff[-1])+pow_2(ff[-2]));


		return ff;
	}

//	inline void _init_h()
//	inline void _init_Bbar()
	inline void _init_K(){
		switch (res) {
			case RES_LL:
				_K_LL_V.init(*h_V,qT,mu,LM,cut_off,Lambda);
				_K_LL_S.init(*h_S,qT,mu,LM,cut_off,Lambda);
				K_V	= &_K_LL_V;
				K_S	= &_K_LL_S;
				break;
			case RES_LLO:
				_K_LLO_V.init(*h_V,qT,Lmu,&etaB_V);
				_K_LLO_S.init(*h_S,qT,Lmu,&etaB_S);
				K_V	= &_K_LLO_V;
				K_S	= &_K_LLO_S;
				break;
			default:
				cerr	<<"******************************************************************************************************"<<endl;
				cerr	<<"***** _TMD_PDF_Integrand::_init_K() WRONG Resummation:"<<res<<" != "<<RES_LL<<" or "<<RES_LLO<<" *****"<<endl;
				cerr	<<"******************************************************************************************************"<<endl;
				exit(1);
				break;
		}
	}

	inline void init(
			ENUM_B_FUNCS B
			,NS_LHAFUNC::VectorPDF &pdf
			, const double *xT, const double *mu, const int *nf, const double *as, const double *LT
			,int order, bool improved, bool exponent, CUTOFF cutoff
			,const double *Lambda, const double *LM
			,const double Gamma_Cusp_3_Pade_Factor
			,const double Anomaly_Exponent_F_3_0_Factor
			);

public:

	inline void init(
			ENUM_B_FUNCS B
			,NS_LHAFUNC::VectorPDF &pdf
			, const double *qT, const double *xT, const double *mu, const int *nf, const double *as, const double *LT, const double *Lmu, const double *q_star
			,RES res, int order, POWER_COUNTING pc, bool exponent, CUTOFF cutoff, const double *acc
			,const double *Lambda, const double *LM
			,const double Gamma_Cusp_3_Pade_Factor
			,const double Anomaly_Exponent_F_3_0_Factor
			){
		_TDD_PDF_Integrand::init(B
			,pdf
			,xT,mu,nf,as,LT
			,order,pc,exponent,cutoff
			,Lambda,LM
			,Gamma_Cusp_3_Pade_Factor,Anomaly_Exponent_F_3_0_Factor);

		this->res	= res;
		this->qT	= qT;
		this->Lmu	= Lmu;
		this->q_star= q_star;
		this->acc	= acc;

		_init_K();

	}

	inline void set_xi_qT_mu(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ){
//		NS_Utils::DOUT	dout("_TMD_PDF_Integrand::set_xi_qT_mu","IQ,IQ");
		_TDD_PDF_Integrand::set_xi_xT_mu(IG,IQ);

		etaB_V	= C_F*GAMMA_0*(*as)*(*LM);
		etaB_S	= C_A*GAMMA_0*(*as)*(*LM);

		K_V->update_mu();
		K_S->update_mu();

	}


	_TMD_PDF_Integrand()
	:_TDD_PDF_Integrand()
	,_BTilde(NB,Pol_ae(5,0.))
	,_BTilde_it(),_BTilde_beg(_BTilde.begin()),_BTilde_end(_BTilde.end()),_BTildeG(_BTilde_beg+6),_BTildeG1(_BTilde_end-2),_BTildeG2(_BTilde_end-1)
	,_K_LL_V(),_K_LL_S()
	,_K_LLO_V(),_K_LLO_S()
	,qT(NULL),Lmu(NULL),q_star(NULL)
	,etaB_V(),etaB_S()
	,acc()
	,res()
	,K_V(NULL),K_S(NULL){

	}
	_TMD_PDF_Integrand(const _TMD_PDF_Integrand &O)
	:_TDD_PDF_Integrand(O)
	,_BTilde(NB,Pol_ae(5,0.))
	,_BTilde_it(),_BTilde_beg(_BTilde.begin()),_BTilde_end(_BTilde.end()),_BTildeG(_BTilde_beg+6),_BTildeG1(_BTilde_end-2),_BTildeG2(_BTilde_end-1)
	,_K_LL_V(),_K_LL_S()
	,_K_LLO_V(),_K_LLO_S()
	,qT(O.qT),Lmu(O.Lmu),q_star(O.q_star)
	,etaB_V(O.etaB_V),etaB_S(O.etaB_S)
	,acc(O.acc)
	,res(O.res)
	,K_V(NULL),K_S(NULL){
		_BTilde	= O._BTilde;
		if( (O.K_V!=NULL)&&(O.K_S!=NULL) ) _init_K();
	}


	_TMD_PDF_Integrand& operator=(const _TMD_PDF_Integrand &O){
		_TDD_PDF_Integrand::operator =(O);

		_BTilde	= (O._BTilde);

		qT		= (O.qT);
		Lmu		= (O.Lmu);
		q_star	= (O.q_star);
		etaB_V	= (O.etaB_V);
		etaB_S	= (O.etaB_S);
		acc		= (O.acc);

		res	= (O.res);

		if( (O.K_V!=NULL)&&(O.K_S!=NULL) ){
			_init_K();
		}else{
			K_V	= (NULL);
			K_S	= (NULL);
		}

		return *this;
	}

	~_TMD_PDF_Integrand(){
	}


	inline double* integrand_BTilde(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac=1.){
//		NS_Utils::DOUT dout("_TMD_PDF_Integrand::integrand_BTilde");
		_update_BTilde(IG,IQ);
		return _fill_from_BTilde<false>(ff,Jac);
	}

	inline double* integrand_BTilde_over_phi(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac=1.){
//		NS_Utils::DOUT dout("_TMD_PDF_Integrand::integrand_BTilde_over_phi");
		_update_BTilde(IG,IQ);
		return _fill_from_BTilde<true>(ff,Jac);
	}

	inline double* integrand_BTilde_times_xi(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac=1.){
//		NS_Utils::DOUT dout("_TMD_PDF_Integrand::integrand_BTilde_times_xi");
		return integrand_BTilde(IG,IQ,ff,Jac*(IQ.x));
	}

	template< E_INT e_int >
	inline double* integrand(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac){
		// INT_B
		return integrand_B(IG,IQ,ff,Jac);
	}
};


template<  >
inline double* _TMD_PDF_Integrand::integrand<INT_XI_B>(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac){
	return integrand_B_times_xi(IG,IQ,ff,Jac);
}

template<  >
inline double* _TMD_PDF_Integrand::integrand<INT_B_over_Phi>(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac){
	return integrand_B_over_phi(IG,IQ,ff,Jac);
}

template<  >
inline double* _TMD_PDF_Integrand::integrand<INT_Btilde>(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac){
	return integrand_BTilde(IG,IQ,ff,Jac);
}

template<  >
inline double* _TMD_PDF_Integrand::integrand<INT_XI_Btilde>(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac){
	return integrand_BTilde_times_xi(IG,IQ,ff,Jac);
}

template<  >
inline double* _TMD_PDF_Integrand::integrand<INT_Btilde_over_Phi>(const Ibar_G_Parameter &IG,const Ibar_Q_Parameter &IQ, double ff[NB], double Jac){
	return integrand_BTilde_over_phi(IG,IQ,ff,Jac);
}

}  // NS_TDPDF_Integrand

}  // NS_TDPDF

}  // namespace CuTe
#endif
