#include "charges.h"

using namespace CuTe::NS_CHARGES;

using std::cout;
using std::endl;


std::ostream& operator<<( std::ostream& os,const CKM_MATRIX& ckm ){
//	std::stringstream ss;
	os	<<*(ckm.CVud)<<" "
		<<*(ckm.CVus)<<" "
		<<*(ckm.CVub)<<" "
		<<*(ckm.CVcd)<<" "
		<<*(ckm.CVcs)<<" "
		<<*(ckm.CVcb)<<" "
		<<*(ckm.CVtd)<<" "
		<<*(ckm.CVts)<<" "
		<<*(ckm.CVtb)<<" "
		;
//	if(os!=cout) cout	<<"operator<<:"<<ss.str()	<<endl;
//	os << ss.str();
	return os;
}


std::istream& operator>>( std::istream& is,CKM_MATRIX& ckm ){
	is	>>*(ckm.Vud)
		>>*(ckm.Vus)
		>>*(ckm.Vub)
		>>*(ckm.Vcd)
		>>*(ckm.Vcs)
		>>*(ckm.Vcb)
		>>*(ckm.Vtd)
		>>*(ckm.Vts)
		>>*(ckm.Vtb)
		;
//	cout	<<"operator>>:"<<ckm	<<endl;
	return is;
}


std::ostream& operator<<( std::ostream& os, const Charge& C){
	using CuTe::NS_enums::NS_Partons::operator<<;
	os	<<"\t";
	for(Parton p(Parton_tbar); p<=Parton_t; ++p)	os	<<"\t"<<p;
	os	<<endl;

	for(Parton q(Parton_t); q>=Parton_tbar; --q){
		os	<<q;
		for(Parton p(Parton_tbar); p<=Parton_t; ++p){
			os	<<"\t"<<(C.is_active(q,p));
		}
		os	<<endl;
	}
	return os;
}


std::ostream& Q_Matrix::to_stream(std::ostream& os, const int nf, const int ntab)const{
	std::string tab;
	for(int i=0; i<ntab; ++i) tab += "\t";

	using CuTe::NS_enums::NS_Partons::operator<<;
	os	<<tab;
	for(Parton p(-nf); p<=Parton(nf); ++p)	os	<<"\t"<<p;
	os	<<endl;

	for(Parton q(-nf); q<=Parton(nf); ++q){
		os	<<tab<<q;
		for(Parton p(-nf); p<=Parton(nf); ++p){
			os	<<"\t"<<C.is_active(q,p);
//			os	<<"\t"<<(is_active(q)&&is_active(p));
		}
		os	<<endl;
	}
	return os;
}





template < >
Matrix< 2, complex<double> > Pauli_Matrices::Pauli_i< Pauli_Matrices::Pauli_1 >(){
	Matrix<2,complex<double> > M(complex<double>(0.,0.));
	M[0][1]=complex<double>(1.,0.);
	M[1][0]=complex<double>(1.,0.);
	return M;
}

template < >
Matrix< 2, complex<double> > Pauli_Matrices::Pauli_i< Pauli_Matrices::Pauli_2 >(){
	Matrix<2,complex<double> > M(complex<double>(0.,0.));
	M[0][1]=complex<double>(0,-1.);
	M[1][0]=complex<double>(0,+1.);
	return M;
}

template < >
Matrix< 2, complex<double> > Pauli_Matrices::Pauli_i< Pauli_Matrices::Pauli_3 >(){
	Matrix<2,complex<double> > M(complex<double>(0.,0.));
	M[0][0]=complex<double>(+1.,0.);
	M[1][1]=complex<double>(-1.,0.);
	return M;
}

template < >
Matrix< 2, complex<double> > Pauli_Matrices::Pauli_i< Pauli_Matrices::Pauli_Plus >(){
	Matrix<2,complex<double> > M(Pauli_i<Pauli_1>());
	M += (Pauli_i<Pauli_2>()*complex<double>(0,+1.));
	M *= 0.5;
	return M;
}

template < >
Matrix< 2, complex<double> > Pauli_Matrices::Pauli_i< Pauli_Matrices::Pauli_Minus >(){
	Matrix<2,complex<double> > M(Pauli_i<Pauli_1>());
	M -= (Pauli_i<Pauli_2>()*complex<double>(0,+1.));
	M *= 0.5;
	return M;
}

const Matrix< 2, complex<double> > CuTe::NS_CHARGES::Charge::pauli_three = Pauli_Matrices::Pauli_i<Pauli_Matrices::Pauli_3>();
const Matrix< 2, complex<double> > CuTe::NS_CHARGES::Charge::pauli_plus  = Pauli_Matrices::Pauli_i<Pauli_Matrices::Pauli_Plus>();

