#ifndef FILE_PATH_H
#define FILE_PATH_H

#include <iostream>
#include <sstream>


namespace NS_File_Path {
using std::cout;
using std::endl;
using std::istream;
using std::ostream;
using std::string;
using std::stringstream;

static const char del_ext	= '.';
static const char del_root	= '/';
static const char del_folder= '/';
static const char del_home	= '~';
static const char del_var	= '.';
static const char del_back	= '.';
enum e_path_sort { e_root, e_home, e_var, e_file};

inline ostream& operator<<(ostream& os, const e_path_sort e){
	switch (e) {
	case e_root:
		os<<"root("<<del_root<<")";
		break;
	case e_home:
		os<<"home("<<del_home<<")";
		break;
	case e_var:
		os<<"var("<<del_var<<")";
		break;
	case e_file:
		os<<"file()";
		break;
	default:
		break;
	}
	return os;
}

class File_Name{
public:

	string full;
	string name,ext;

	static const size_t get_name_of(const string& s, string& name){
		size_t pos = s.find_last_of(del_ext);
		if(pos==string::npos) name = s;
		else name = s.substr(0,pos);
		return pos;
	}

	static const string get_name_of(const string& s){
		string name;
		get_name_of(s,name);
		return name;
	}

	static const size_t get_ext_of(const string& s, string& ext){
		size_t pos = s.find_last_of(del_ext);
		if(pos==string::npos) ext = string();
		else ext = s.substr(pos+1);
		return pos;
	}

	static const string get_ext_of(const string& s){
		string ext;
		get_ext_of(s,ext);
		return ext;
	}

protected:

public:
	File_Name()
	:full()
	,name(),ext(){
	}

	File_Name(const string& full_)
	:full(full_)
	,name(get_name_of(full)),ext(get_ext_of(full)){
	}

	File_Name(const string& name_, const string& ext_)
	:full(name_ + del_ext + ext_)
	,name(name_),ext(ext_){
	}

	explicit File_Name(const File_Name& name_, const string& ext_)
	:full(name_.name + del_ext + ext_)
	,name(name_.name),ext(ext_){
	}

	explicit File_Name(const string& name_, const File_Name& ext_)
	:full(name_ + del_ext + ext_.ext)
	,name(name_),ext(ext_.ext){
	}

	explicit File_Name(const File_Name& name_, const File_Name& ext_)
	:full(name_.name + del_ext + ext_.ext)
	,name(name_.name),ext(ext_.ext){
	}



	const string properties()const{
		stringstream ss;
		ss	<<"File_Name:"	<<endl
			<<"\t\t"<<full	<<endl
			<<"\t\t"<<"name:\t"<<name	<<endl
			<<"\t\t"<<"ext:\t"<<ext	<<endl;
		return ss.str();
	}
};

inline ostream& operator<<(ostream& os, const File_Name& fn){
	os<<fn.full;
	return os;
}


class File_Path{
public:

	e_path_sort path_sort;
	string full;
	string dir;
	File_Name file;

	static const e_path_sort get_path_sort_of(const string& s){
		char first = s[0];
		char second = s[1];
		char third = s[2];
		e_path_sort result(e_file);

		switch (first) {
		case del_root:
			result = e_root;
			break;
		case del_home:
			if(second==del_folder)	result = e_home;
//			result = Not_a_Path;
			break;
		case del_var:
			if(second==del_folder)	result = e_var;
			else if( (second==del_back)&&(third==del_folder) )	result = e_var;
//			result = Not_a_Path;
			break;
		default:
//			result = Not_a_Path;
			break;
		}
		return result;
	}

	static const size_t get_dir_of(const string& s, string& dir){
		size_t pos = s.find_last_of(del_folder);
		if(pos==string::npos) dir = string(1,del_var);
		else	dir = s.substr(0,pos);
		return pos;
	}

	static const string get_dir_of(const string& s){
		string dir;
		get_dir_of(s,dir);
		return dir;
	}

	static const size_t get_file_of(const string& s, string& file){
		size_t pos = s.find_last_of(del_folder);
		if(pos==string::npos) file = s;
		else file = s.substr(pos+1);
		return pos;
	}

	static const string get_file_of(const string& s){
		string file;
		get_file_of(s,file);
		return file;
	}

protected:

public:
	File_Path()
	:path_sort(e_file)
	,full()
	,dir(),file(){
	}

	File_Path(const string& full_)
	:path_sort(get_path_sort_of(full_))
	,full(full_)
	,dir(get_dir_of(full)),file(get_file_of(full)){
	}

	File_Path(const string& dir_, const File_Name& file_)	//=:C1
//	implicit_1 (string,string)	ok
	:path_sort(get_path_sort_of(dir_))
	,full(dir_ + del_folder + file_.full)
	,dir(dir_),file(file_){
	}

	File_Path(const File_Path& dir_, const File_Name& file_)	//=:C2
//	implicit_1 (F_P,string)		ok
//	implicit_1 (string,F_N)		ruled out by C1
//	implicit_2 (string,string)	ruled out by C1
	:path_sort(dir_.path_sort)
	,full(dir_.dir + del_folder + file_.full)
	,dir(dir_.dir),file(file_){
	}

	explicit File_Path(const File_Path& dir_, const File_Path& file_)	//=:C3
//	implicit_1 (F_P,string)		!
//	implicit_1 (string,F_P)		!
//	implicit_2 (string,string)	ruled out by C1
	:path_sort(dir_.path_sort)
	,full(dir_.dir + del_folder + file_.file.full)
	,dir(dir_.dir),file(file_.file){
	}

	explicit File_Path(const string& dir_, const File_Path& file_)	//=:C4
//	implicit_1 (string,string)	!
	:path_sort(get_path_sort_of(dir_))
	,full(dir_ + del_folder + file_.file.full)
	,dir(dir_),file(file_.file){
	}

	File_Path(const string& dir_, const string& name, const string& ext)
	:path_sort(get_path_sort_of(dir_))
	,full(dir_ + del_folder + name + del_ext + ext)
	,dir(dir_),file(name,ext){
	}

	explicit File_Path(const File_Path& dir_, const File_Path& name, const string& ext)
	:path_sort(dir_.path_sort)
	,full(dir_.dir + del_folder + name.file.name + del_ext + ext)
	,dir(dir_.dir),file(name.file.name,ext){
	}

	explicit File_Path(const File_Path& dir_, const File_Path& name, const File_Path& ext)
	:path_sort(dir_.path_sort)
	,full(dir_.dir + del_folder + name.file.name + del_ext + ext.file.ext)
	,dir(dir_.dir),file(name.file.name,ext.file.ext){
	}

	const string properties()const{
		stringstream ss;
		ss	<<"File_Path:"	<<endl
			<<"\t"<<full	<<endl
			<<"\t"<<"is "<<path_sort	<<endl
			<<"\t"<<"dir:\t"<<dir	<<endl
			<<"\t"<<"file:\t"<<file.properties()	<<endl;
		return ss.str();
	}

	inline const char* c_str()const{
		return full.c_str();
	}

	inline operator string()const{
		string s(full);
		return s;
	}

	inline operator string&(){
		return full;
	}
};

inline ostream& operator<<(ostream& os, const File_Path& fp){
	os<<fp.full;
	return os;
}

inline istream& operator>>(istream& is, File_Path& fp){
	string s;
	is >> s;
	fp = File_Path(s);
	return is;
}

}  // namespace NS_File_Path

#endif
