#include "param.h"
#include "Kinematics.h"

#ifdef __cplusplus
extern "C" {
#endif
	#include "gsl/gsl_errno.h"
	#include "gsl/gsl_roots.h"
#ifdef __cplusplus
}
#endif

using CuTe::NS_CS_Params::Q_star;
using CuTe::NS_CS_Params::CS_Parameter;
using CuTe::NS_CS_Params::MU_star;
using CuTe::NS_CS_Params::PDFset;

void PDFset::change_sign(const SOPS_CS_Params& OP){
	switch (OP.Process.col()) {
	case COL_PP:
		sign[0]	= +1;
		sign[1]	= +1;
		break;
	case COL_PbarP:
		sign[0]	= +1;
		sign[1]	= -1;
		break;
	case COL_PbarPbar:
		sign[0]	= -1;
		sign[1]	= -1;
		break;
	default:
		std::cerr	<<"PDFset::init(op) Wrong Col Argument ->exit(1)"<<endl;
		exit(1);
		break;
	}

}

void PDFset::change_PDF(const SOPS_CS_Params &OP){
	_AS	= lha_pdfset.init(OP.PDF.PDFset(),OP.PDF.PDF_Error(),OP.PDF.PDF_member(),OP.PDF.LHAPDF_verbosity());
	lha_pdfset.update(pdfs,sign);
	if(OP.PDF.PDF_Dummy_a_b.is_set) set_dummys(OP.PDF.PDF_Dummy_a_b());
}

void PDFset::change_PDF(const SOPS_CS_Params &OP,LHAPDF::PDFSet* Set){
	_AS	= lha_pdfset.init(Set,OP.PDF.PDF_Error(),OP.PDF.PDF_member(),OP.PDF.LHAPDF_verbosity());
	lha_pdfset.update(pdfs,sign);
}

void PDFset::change_PDF(NS_LHAFUNC::OWN_LHAPDF* pdf){
	_AS	= lha_pdfset.init(pdf);
	lha_pdfset.update(pdfs,sign);
}

void PDFset::init(const SOPS_CS_Params &OP){
	if(OP.PDF.PDFset.is_set){
		change_sign(OP);
		change_PDF(OP);
	}else{
		std::cerr	<<"PDFset::init(const SOPS_CS_Params &OP) OP.PDF.PDFset is unset -> skip initialization"<<endl;
	}
}

void PDFset::init(const SOPS_CS_Params &OP,LHAPDF::PDFSet* Set){
	change_sign(OP);
	change_PDF(OP,Set);
}

void PDFset::init(const SOPS_CS_Params &OP,NS_LHAFUNC::OWN_LHAPDF* pdf){
	change_sign(OP);
	change_PDF(pdf);
}

void PDFset::calc_errors(vector<double>& results, const vector<double>& values){
//		NS_Utils::DOUT dout("PDFset::calc_errors");
	lha_pdfset.set_uncertainty(values);
	using namespace NS_enums::NS_RESULTS;
	results[ER_erg_result]		= +lha_pdfset.get_uncertainty_central();
	results[ER_erg_PDF_DeltaP]	= +lha_pdfset.get_uncertainty_errplus();
	results[ER_erg_PDF_DeltaM]	= -lha_pdfset.get_uncertainty_errminus();
	results[ER_erg_PDF_Delta]	= +lha_pdfset.get_uncertainty_errsymm();
	results[ER_erg_PDF_Scale]	= +lha_pdfset.get_uncertainty_scale();
}

void MU_star::init(const PROD prod, const double M, const OPS_Scales& op_s,const NS_LHAFUNC::A_strong* as){
	//		NS_Utils::DOUT dout("MU_star::init","op,as");
	_MUFAC	= (op_s.scale_variation());
	errors	= (_MUFAC>0);

	if(op_s.fix_mu_to_value.is_set){
		mu	= op_s.fix_mu_to_value();
		to_calc	= false;
	}else{
		if((op_s.fix_mu_to_const.is_set)&&(op_s.fix_mu_to_const()!=ENUM_MU_MU_STAR)){
			switch (op_s.fix_mu_to_const()) {
			case ENUM_MU_M:
				mu	= M;
				to_calc	= false;
				break;
			case ENUM_MU_QT:
				to_calc = true;
				q_star	= 0.;
				break;
			case ENUM_MU_QSS:
				to_calc	= false;
				{
					NS_CS_Params::Q_star QS(as,M,C_A*GAMMA_0);
					mu	= QS();
				}
				break;
			case ENUM_MU_QSV:
				to_calc	= false;
				{
					NS_CS_Params::Q_star QS(as,M,C_F*GAMMA_0);
					mu	= QS();
				}
				break;
			default:
				std::cerr	<<"***************************************************************"<<endl
				<<"**** MU_star::init WRONG fix_mu_to_const Option -> exit(1) ****"<<endl
				<<"***************************************************************"<<endl;
				exit(1);
				break;
			}
		}else{
//		to_calc	= true; <_in change
			if(op_s.q_star.is_set){
				change_qstar(op_s.q_star());
			}else{
				change_qstar(prod,M,as);
			}
		}
	}
	if(errors){
		//			dout()	<<"errors"<<endl;
		mus.resize(2);
		if(!to_calc){
			mus[0] = mu/_MUFAC;
			mus[1] = mu*_MUFAC;
		}
	}else{
		//			dout()	<<"No errors"<<endl;
		mus.resize(0);
	}
}


void MU_star::change_qstar(const double qstar){
	this->q_star = qstar;
	to_calc	= true;
}

void MU_star::change_qstar(const PROD prod, const double M,const NS_LHAFUNC::A_strong* as){
	double G_0_B	= GAMMA_0;
	switch (prod) {
	case PROD_Higgs:
	case PROD_H_RS:
		G_0_B*=C_A;
		break;
	default:
		G_0_B*=C_F;
		break;
	}

	Q_star QS(as,M,G_0_B);
	change_qstar(QS());
}


int Q_star::NSolve(double& q_star, const double estimate)const{
	//		cout << "Q_star::NSolve("<<estimate<<","<<gen<<")" << endl;
	//		debug.resize(3);
	int status;
	int iter = 0;
	const gsl_root_fdfsolver_type *T;
	gsl_root_fdfsolver *s;
	double x0;
	double x = estimate;
	gsl_function_fdf FDF;

	FDF.f = &f;
	FDF.df = &df;
	FDF.fdf = &fdf;
	FDF.params = (void*)this;

	T = gsl_root_fdfsolver_steffenson;
	//		T = gsl_root_fdfsolver_newton;
	s = gsl_root_fdfsolver_alloc (T);
	gsl_root_fdfsolver_set (s, &FDF, x);
	do{
		iter++;
		status = gsl_root_fdfsolver_iterate (s);
		x0 = x;
		x = gsl_root_fdfsolver_root (s);
		status = gsl_root_test_delta (x, x0, 0, gen);

		if(status!=GSL_CONTINUE && status!=GSL_SUCCESS){
			//				cout << "Q_star::NSolve()\t";
			NS_Utils::GSL_ERROR_CODE("Q_star::NSolve()",status);
		}
	}while (status == GSL_CONTINUE && iter < MAX_ITER);
	q_star=x;
	gsl_root_fdfsolver_free (s);
	return status;
}


void CS_Parameter::update_qT_mu(const Kinematics *Kin,const double mu){
	update_qT_mu(Kin->qT,mu);
}

bool MU_star::calc_min_max(const double centre, const double value0, const double value1, double &min, double &max)const{
	return NS_Utils::parabola_min_max(mus[0],value0,mu,centre,mus[1],value1,min,max);
}

bool MU_star::calc_error_MP(const double centre, const double value0, const double value1, double &E_M, double &E_P)const{
	bool ok(calc_min_max(centre,value0,value1,E_M,E_P));
	E_M	-= centre;
	E_P	-= centre;
	return ok;
}

bool MU_star::calc_error(const vector<double> &values_c_0_1, vector<double> &results)const{
	using NS_enums::NS_RESULTS::ER_erg_mu_P;
	using NS_enums::NS_RESULTS::ER_erg_mu_M;

	return calc_error_MP(values_c_0_1[0],values_c_0_1[1],values_c_0_1[2],results[ER_erg_mu_M],results[ER_erg_mu_P]);
}
