#ifndef test_H_
#define test_H_

#define TEST

#ifdef TEST

#include "CuTe.h"
#include "APSFunctions.h"
#include "Res_IQI.h"
#include "Res_Exponents.h"
#include "Res_K_Integral.h"


namespace NS_test{
using std::ofstream;
using std::complex;
using std::endl;
using std::stringstream;
using namespace CuTe;

using NS_CS_Params::CS_Parameter;
using NS_CS_Params::PDFset;
using namespace NS_enums;
using NS_Anomalous_Dimension::AD_Beta;
using namespace NS_Utils;

using NS_OPTIONS::File_Path;


void test_APSF(SOPS_CuTe& OP){
	NS_APSF::APSF_Parameter p(2);
	p.set_x(0.1);
	p.update(0.6);
	cout << NS_APSF::APSF<NS_APSF::e_q,NS_APSF::e_q,0>(0.5,0.1,p,5)<<endl;
	cout << NS_APSF::APSF<NS_APSF::e_q,NS_APSF::e_q,1>(0.5,0.1,p,5)<<endl;
	cout << NS_APSF::APSF<NS_APSF::e_q,NS_APSF::e_q,2>(0.5,0.1,p,5)<<endl;

}

void test_lnk(SOPS_CuTe& OP){
	NS_Resumed::G_exp_Pol gexp;
	NS_Polynom::Pol< complex< double > > kexp;

	int order_ae;
	NS_PC::POWER_COUNTING pc;
	bool exponent;
	NS_PROD::PROD prod;

	const int nf(5);
	const double as(0.1);
	const double mu(10.);
	const double M(100.);
	const double LM(2.*log(M/mu));

	prod	= NS_PROD::PROD_Z;
	pc		= NS_PC::PC_NAIVE;
	for(order_ae=0;order_ae<=2; ++order_ae){
		exponent = false;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout <<gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;

		exponent = true;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout << gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;
	}

	pc		= NS_PC::PC_STD;
	for(order_ae=0;order_ae<=2; ++order_ae){
		exponent = false;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout << gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;

		exponent = true;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout << gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;
	}

	pc		= NS_PC::PC_IMP;
	for(order_ae=0;order_ae<=5; ++order_ae){
		exponent = false;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout << gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;

		exponent = true;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout << gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;
	}


	prod	= NS_PROD::PROD_Higgs;
	pc		= NS_PC::PC_NAIVE;
	for(order_ae=0;order_ae<=2; ++order_ae){
		exponent = false;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout << gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;

		exponent = true;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout << gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;
	}

	pc		= NS_PC::PC_STD;
	for(order_ae=0;order_ae<=2; ++order_ae){
		exponent = false;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout << gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;

		exponent = true;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout << gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;
	}

	pc		= NS_PC::PC_IMP;
	for(order_ae=0;order_ae<=5; ++order_ae){
		exponent = false;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout << gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;

		exponent = true;
		gexp.init(order_ae,pc,exponent,&nf,&as,&LM,prod,1.,0.);
		gexp.update_mu(kexp);
		cout << gexp.properties()<<endl;
		cout <<"\t"<<"g = "<<gexp<<endl;
		cout <<"\t"<<"k = "<<kexp<<endl;
		cout <<endl;
	}
}

inline void test_K(SOPS_CuTe& OP){
	File_Path K_FileName(OP.get_infilename());
	cout	<<"I_file: "<<K_FileName<<endl;
	K_FileName	= File_Path(K_FileName.dir,K_FileName.file.name+"_K","out");
	cout	<<"K_file: "<<K_FileName<<endl;

	File_Path k_FileName(OP.get_infilename());
	k_FileName	= File_Path(k_FileName.dir,k_FileName.file.name+"_k","out");
	cout	<<"k_file: "<<k_FileName<<endl;


	std::vector<double>	qT_vals;
	NS_Utils::insert_N_values(OP.Plot.qT(),OP.Plot.qT_N(),qT_vals);

	cout	<<"qT values: "<<qT_vals<<endl;


	PDFset pdfset;
	pdfset.init(OP);

	NS_CS_Params::CS_Parameter_LL	p;
	p.init(OP,pdfset[0]);

	NS_CS_Params::MU_star mu_star;
	mu_star.init(OP,pdfset.as_central());

	NS_Resumed::G_exp_Pol gexp;
	gexp.init(p.order,p.pc,p.exponent,&(p.nf),&(p.as),&(p.LM),p.prod,p.Gamma_Cusp_3_Pade_Factor,p.Anomaly_Exponent_F_3_0_Factor);

	double qT;

	NS_Resumed::K_LL k_ll;
	k_ll.init(gexp,&qT,&(p.mu),&(p.LM),p.cutoff,&(p.Lambda));

	std::vector<double>::const_iterator	itr_qT(qT_vals.begin());
	const std::vector<double>::const_iterator	end(qT_vals.end());

	ofstream of_k(k_FileName.full.c_str());
	ofstream of_K(K_FileName.full.c_str());


	while(itr_qT!=end){
		qT	= *itr_qT;
		p.update_qT_mu(qT,mu_star(qT));

		k_ll.update_mu();

		of_k	<<qT	<<"\t"<<p.mu;
		of_K	<<qT	<<"\t"<<p.mu;

		for(int i=0; i<=k_ll.get_kexp().order(); ++i){
			of_k	<<"\t"<<k_ll.get_kexp()[i].real();
		}

		for(int i=0; i<k_ll.get_Kn().size(); ++i){
			of_K	<<"\t"<<k_ll.get_Kn()[i];
		}



		of_k	<<endl;
		of_K	<<endl;

		++itr_qT;
	}

	of_k.close();
	of_K.close();

}

inline void test_Ibar_conv(SOPS_CuTe& OP){
	double xi	= 0.5;
	double  z	= 0.8;

	int a	=1;
	int b	=1;

	double phi_xi	= pow(xi,a)*pow(1.-xi,b);
	double phi_z	= pow(z,a)*pow(1.-z,b);

	int nf	= 5;


	int order		= OP.Scheme.order();
	int order_a		= order;
	POWER_COUNTING pc			= OP.Scheme.Power_Counting();
	const bool improved	= (pc==PC_IMP);

	if(improved){
		if(OP.Scheme.order_epsilon.is_set) order= OP.Scheme.order_epsilon();
		else order= 2*order +1;
	}

	NS_Ibar::Ibar_G_Parameter IGP;
	IGP.init(order,improved,order_a);
	IGP.set_x(xi);
	IGP.update(z);

//	NS_Polynom::Pol< NS_Polynom::Pol<double> > P(5,NS_Polynom::Pol<double>(5,0.));
//
//	NS_Ibar::Ibar<NS_Ibar::E_Ibar_G,NS_Ibar::e_g,2,0>(P,phi_z,phi_xi,&IGP,nf);
//
//	cout	<<"ibarphi:"<<endl<<P<<endl;

	NS_Ibar::Ibar_ij_conv_f ibarphi;
//	ibarphi.init(order);
//	ibarphi.update<NS_Ibar::E_Ibar_G,NS_Ibar::e_g>(phi_z,phi_xi,&IGP,nf);

	cout	<<"ibarphi:"<<endl<<ibarphi<<endl;


}

using namespace NS_enums::NS_Transition_Function;

void test_Stepfunction(SOPS_CuTe& OP){

	File_Path OutFileName(OP.get_infilename());
	cout	<<"Ifile: "<<OutFileName<<endl;
	OutFileName	= File_Path(OutFileName.dir,OutFileName.file.name,"out");
	cout	<<"Ofile: "<<OutFileName<<endl;

	std::vector<double>	qT_vals;
	NS_Utils::insert_N_values(OP.Plot.qT(),OP.Plot.qT_N(),qT_vals);

	cout	<<"qT values: "<<qT_vals<<endl;


	vector<ENUM_Transition_Function_Type> types(4,TFT_NO);
	types[1]	= TFT_LOG;
	types[2]	= TFT_ERF;
	types[3]	= TFT_GPZ;


	vector<TransitionFunction> SF_V(types.size(),TransitionFunction());
	const vector<TransitionFunction>::iterator SF_begin(SF_V.begin());
	const vector<TransitionFunction>::const_iterator SF_end(SF_V.end());
	vector<TransitionFunction>::iterator SF_I;

//	const double M(OP.Process.M());
//	vector<double> lr_r;
//	lr_r.push_back(0.5*M);
//	lr_r.push_back(0.75*M);
//	lr_r.push_back(0.05);

	ofstream of(OutFileName.full.c_str());

	of<<"#qT";
	for(int i=0; i<types.size();++i){
		cout<<"\t"<<types[i];
		SF_V[i].init(&OP,types[i],OP.Scheme.transition_arg());
		of<<"\t"<<types[i];
	}
	cout<<endl;
	for(int i=0; i<types.size();++i){
		of<<"\tbar_"<<types[i];
	}
	of<<endl;

	std::vector<double>::const_iterator	qT(qT_vals.begin());
	const std::vector<double>::const_iterator	end(qT_vals.end());

	while(qT!=end){

		of<<*qT;

		SF_I	= SF_begin;
		while(SF_I!=SF_end){
			of<<"\t"<<(*SF_I).f(*qT);
			++SF_I;
		}

		SF_I	= SF_begin;
		while(SF_I!=SF_end){
			of<<"\t"<<(*SF_I).barf(*qT);
			++SF_I;
		}

		of<<endl;

		++qT;
	}


	of.close();
}

void test_H(SOPS_CuTe& OP){

	File_Path OutFileName(OP.get_infilename());
	cout	<<"Ifile: "<<OutFileName<<endl;
	OutFileName	= File_Path(OutFileName.dir,OutFileName.file.name,"out");
	cout	<<"Ofile: "<<OutFileName<<endl;

	std::vector<double>	qT_vals;
	NS_Utils::insert_N_values(OP.Plot.qT(),OP.Plot.qT_N(),qT_vals);

	cout	<<"qT values: "<<qT_vals<<endl;


	PDFset pdfset;
	pdfset.init(OP);

	OP.Scheme.PiQuadrat.value =true;
	OP.Scheme.exponent.value =false;

	NS_CS_Params::CS_Parameter_LL	p_LL_pi2;
	NS_CS_Params::CS_Parameter_LL	p_LL;

	p_LL_pi2.init(OP,pdfset[0]);

	const double q_star=p_LL_pi2.calc_qstar();

	OP.Scheme.PiQuadrat.value =false;
	p_LL.init(OP,pdfset[0]);


	NS_Hard::Hardfunc_LL	Hinit;
	NS_Hard::Hardfunc_LL	H_LL(Hinit);
	Hinit.init(&p_LL_pi2);
	NS_Hard::Hardfunc_LL	H_LL_pi2(Hinit);
//	NS_Hard::Hardfunc_MC	H_MC;
//	NS_Hard::Hardfunc_MC	H_MC_pi2;
	NS_Hard::Hardfunc_SQT	H_SQT;
	NS_Hard::Hardfunc_SQT	H_SQT_pi2;

	NS_Hard::MS_Factor		MS_F_NAIVE;

	NS_Hard::MS_Factor		MS_F_VAR;
	NS_Hard::MS_Factor		MS_F_VAR_PI2;
	NS_Hard::MS_Factor		MS_F_FULL;
	NS_Hard::MS_Factor		MS_F_FULL_PI2;

//	NS_Hard::MS_Factor		MS_F2_VAR;
//	NS_Hard::MS_Factor		MS_F2_VAR_PI2;
//	NS_Hard::MS_Factor		MS_F2_FULL;
//	NS_Hard::MS_Factor		MS_F2_FULL_PI2;

	H_LL.init(&p_LL);
//	H_LL_pi2.init(&p_LL_pi2);
//	H_MC.init(&p_LL);
//	H_MC_pi2.init(&p_LL,true);
	H_SQT.init(&p_LL);
	H_SQT_pi2.init(&p_LL,true);

	MS_F_NAIVE.init(&p_LL,NS_MS::MS_NAIVE);

	MS_F_VAR.init(&p_LL,NS_MS::MS_VAR_MU);
	MS_F_VAR_PI2.init(&p_LL,NS_MS::MS_VAR_MU_PI2);
	MS_F_FULL.init(&p_LL,NS_MS::MS_VAR_QT);
	MS_F_FULL_PI2.init(&p_LL,NS_MS::MS_VAR_QT_PI2);

//	MS_F2_VAR.init(&p_LL,NS_MS::MS_VAR);
//	MS_F2_VAR_PI2.init(&p_LL,NS_MS::MS_VAR_PI2);
//	MS_F2_FULL.init(&p_LL,NS_MS::MS_FULL_SUDAKOV);
//	MS_F2_FULL_PI2.init(&p_LL,NS_MS::MS_FULL_SUDAKOV_PI2);

	const double M(p_LL.M);

	std::vector<double>::const_iterator	qT(qT_vals.begin());
	const std::vector<double>::const_iterator	end(qT_vals.end());


	ofstream of(OutFileName.full.c_str());

//	of	<<"#\t\tH_LL\tH_LL_nop2\tH_LL_exp\tH_SQT\tH_SQT_plusO\tH2_LL\tH2_LL_nop2\tH2_LL_exp\tH2_SQT\tH2_SQT_plusO\tU_LL\tU_LL_nop2\tU_LL_exp"<<endl;
	of	<<"#\tqT\tmu";
	of	<<"\tH_LL\tH_LL_pi2";
	of	<<"\tH_SQT\tH_SQT_pi2";
//	of	<<"\tH_MC_VAR\tH_MC_VARPI2";
//	of	<<"\tH_MC_FULL\tH_MC_FULLPI2";
	of	<<"\tf";
	of	<<"\tF_VAR"<<"\tF_VARPI2";
	of	<<"\tF_FULL"<<"\tF_FULLPI2";
	of	<<"\tF_VAR_f"<<"\tF_VARPI2_f";
	of	<<"\tF_FULL_f"<<"\tF_FULLPI2_f";
//	of	<<"\tF2_VAR"<<"\tF2_VARPI2";
//	of	<<"\tF2_FULL"<<"\tF2_FULLPI2";
//	of	<<"\tF2_VAR_f"<<"\tF2_VARPI2_f";
//	of	<<"\tF2_FULL_f"<<"\tF2_FULLPI2_f";
	of	<<endl;

	TransitionFunction stepfun;

	stepfun.init(&OP);

//	stepfun.init_left_right(M/2.,M);
//	stepfun.init_mu_sigma(M,M/2.);
//	stepfun.init_mu_sigma(M/2.,M/2);

	stepfun.to_stream(cout,1);

	while(qT!=end){
		const double mu	= *qT + q_star*exp(-(*qT)/q_star);
		of	<<*qT<<"\t"<<mu;
//		MS_F_NAIVE.update_mu(*qT,mu,mu);
//		cout<<MS_F_NAIVE<<endl;

//		MS_F.to_stream(cout,1);

		const double f(stepfun.f(*qT));

		H_LL.update_mu(mu);
		H_LL_pi2.update_mu(mu);
		H_SQT.update_mu(mu);
		H_SQT_pi2.update_mu(mu);
//		H_MC.update_mu(mu);
//		H_MC_pi2.update_mu(mu);

		of	<<"\t"<<H_LL.of_1()<<"\t"<<H_LL_pi2.of_1();
		of	<<"\t"<<H_SQT.of_1()<<"\t"<<H_SQT_pi2.of_1();
//		of	<<"\t"<<H_MC.of_1()<<"\t"<<H_MC_pi2.of_1();
//		H_MC.update_mu(*qT);
//		H_MC_pi2.update_mu(*qT);
//		of	<<"\t"<<H_MC.of_1()<<"\t"<<H_MC_pi2.of_1();

		of	<<"\t"<<f;


		MS_F_VAR.update_mu(*qT,mu,mu);
		MS_F_VAR_PI2.update_mu(*qT,mu,mu);
		MS_F_FULL.update_mu(*qT,mu,mu);
		MS_F_FULL_PI2.update_mu(*qT,mu,mu);

		of	<<"\t"<<MS_F_VAR.of_1();
		of	<<"\t"<<MS_F_VAR_PI2.of_1();
		of	<<"\t"<<MS_F_FULL.of_1();
		of	<<"\t"<<MS_F_FULL_PI2.of_1();

		MS_F_VAR.update_mu(*qT,mu,mu,f);
		MS_F_VAR_PI2.update_mu(*qT,mu,mu,f);
		MS_F_FULL.update_mu(*qT,mu,mu,f);
		MS_F_FULL_PI2.update_mu(*qT,mu,mu,f);

		of	<<"\t"<<MS_F_VAR.of_1();
		of	<<"\t"<<MS_F_VAR_PI2.of_1();
		of	<<"\t"<<MS_F_FULL.of_1();
		of	<<"\t"<<MS_F_FULL_PI2.of_1();


//		MS_F2_VAR.update_mu(*qT,mu,mu);
//		MS_F2_VAR_PI2.update_mu(*qT,mu,mu);
//		MS_F2_FULL.update_mu(*qT,mu,mu);
//		MS_F2_FULL_PI2.update_mu(*qT,mu,mu);
//
//		of	<<"\t"<<MS_F2_VAR.of_1();
//		of	<<"\t"<<MS_F2_VAR_PI2.of_1();
//		of	<<"\t"<<MS_F2_FULL.of_1();
//		of	<<"\t"<<MS_F2_FULL_PI2.of_1();
//
//		MS_F2_VAR.update_mu(*qT,mu,mu,f);
//		MS_F2_VAR_PI2.update_mu(*qT,mu,mu,f);
//		MS_F2_FULL.update_mu(*qT,mu,mu,f);
//		MS_F2_FULL_PI2.update_mu(*qT,mu,mu,f);
//
//		of	<<"\t"<<MS_F2_VAR.of_1();
//		of	<<"\t"<<MS_F2_VAR_PI2.of_1();
//		of	<<"\t"<<MS_F2_FULL.of_1();
//		of	<<"\t"<<MS_F2_FULL_PI2.of_1();

		of	<<endl;

		++qT;
	}

	of.close();

}

void test_Rms(SOPS_CuTe& OP){

	File_Path OutFileName(OP.get_infilename());
	cout	<<"Ifile: "<<OutFileName<<endl;
	OutFileName	= File_Path(OutFileName.dir,OutFileName.file.name+"_Rms","out");
	cout	<<"Ofile: "<<OutFileName<<endl;

	std::vector<double>	qT_vals;
	NS_Utils::insert_N_values(OP.Plot.qT(),OP.Plot.qT_N(),qT_vals);

	cout	<<"qT values: "<<qT_vals<<endl;


	PDFset pdfset;
	pdfset.init(OP);

	OP.Scheme.PiQuadrat.value =true;
	OP.Scheme.exponent.value =false;

	NS_CS_Params::CS_Parameter_LL	p_LL;
	p_LL.init(OP,pdfset[0]);



	NS_Hard::MS_Factor_test RMS0,RMS1,RMS2;
	RMS0.init(&p_LL,0);
	RMS1.init(&p_LL,1);
	RMS2.init(&p_LL,2);

	std::vector<double>::const_iterator	qT(qT_vals.begin());
	const std::vector<double>::const_iterator	end(qT_vals.end());


	ofstream of(OutFileName.full.c_str());

	of	<<"#\tmu";
	RMS0.FL(of);
	RMS1.FL(of);
	RMS2.FL(of);
	of	<<endl;


	while(qT!=end){

		of	<<*qT;

		RMS0.update(*qT);
		RMS0.result(of);

		RMS1.update(*qT);
		RMS1.result(of);

		RMS2.update(*qT);
		RMS2.result(of);

		of<<endl;

		++qT;
	}

	of.close();

}

void test_Rms2(SOPS_CuTe& OP){

	File_Path OutFileName(OP.get_infilename());
	cout	<<"Ifile: "<<OutFileName<<endl;
	OutFileName	= File_Path(OutFileName.dir,OutFileName.file.name+"_Rms","out");
	cout	<<"Ofile: "<<OutFileName<<endl;

	std::vector<double>	qT_vals;
	NS_Utils::insert_N_values(OP.Plot.qT(),OP.Plot.qT_N(),qT_vals);

	cout	<<"qT values: "<<qT_vals<<endl;


	PDFset pdfset;
	pdfset.init(OP);


	NS_CS_Params::CS_Parameter_LL	p_LL;
	p_LL.init(OP,pdfset[0]);


	NS_Hard::MS_Factor RMS,RMS_PI2;
	RMS.init(&p_LL,NS_MS::MS_VAR_MU);
	RMS_PI2.init(&p_LL,NS_MS::MS_VAR_MU_PI2);

	std::vector<double>::const_iterator	qT(qT_vals.begin());
	const std::vector<double>::const_iterator	end(qT_vals.end());


	ofstream of(OutFileName.full.c_str());

	of	<<"#\tmu";
	of	<<"\tRMS";
	of	<<"\tRMS_PI2";
	of	<<endl;


	while(qT!=end){

		of	<<"\t"<<*qT;

		RMS.update_mu(*qT,*qT,*qT);
		of	<<"\t"<<RMS.of_1();

		RMS_PI2.update_mu(*qT,*qT,*qT);
		of	<<"\t"<<RMS_PI2.of_1();

		of<<endl;

		++qT;
	}

	of.close();

}

inline void test_IQI(SOPS_CuTe& OP){
	NS_Utils::DOUT dout("test_IQI");
	double qT(10);
	double mu(10);
	double y(1);
	double xx[2];
	xx[0]=0.2; xx[1]=0.8;


	PDFset pdfset;
	pdfset.init(OP);

	NS_CS_Params::CS_Parameter_LL p;
	p.init(OP,pdfset[0]);
	p.update_qT_mu(qT,mu);

	cout	<<"\t"<<"sS="<<sqrt(p.S)	<<"\t"<<"M="<<p.M	<<endl;
	cout	<<"\t"<<"qT="<<qT	<<"\t"<<"mu="<<p.mu	<<"\t"<<"y="<<y1	<<endl;

	Kinematics Kin;
	Kin.init(sqrt(p.S),p.M);
	Kin.set_qT_y_get_ok(qT,y);
	Kin.update_zzb_get_OK(xx);


	NS_Resumed::Ibar_Parameter_Set IPS;
	IPS.init(p.prod,p.order,p.improved_pc(),p.order_as);
	IPS.set_y(&Kin);
	IPS.update_zz(&Kin);


	NS_Resumed::IQI iqi;

	iqi.init(p.prod,p.order,p.improved_pc(),p.pdfs,&p.mu,&p.as,&p.nf,p.ckm,p.Theta_W,p.active_flv_charges);
	iqi.set_y(IPS);
	iqi.update_zz(IPS);


}

inline void test_QDCS(SOPS_CuTe& OP){
	OP.Scheme.resummation.set(RES_MATCHED);
	NS_Utils::DOUT dout("test_IQI");
	double qT(10);
	double mu(10);
	double y(1);
	double xx[2];
	xx[0]=0.2; xx[1]=0.8;


	PDFset pdfset;
	pdfset.init(OP);

	NS_CS_Params::CS_Parameter_LL p;
	p.init(OP,pdfset[0]);
	p.update_qT_mu(qT,mu);

	cout	<<"\t"<<"sS="<<sqrt(p.S)	<<"\t"<<"M="<<p.M	<<endl;
	cout	<<"\t"<<"qT="<<qT	<<"\t"<<"mu="<<p.mu	<<"\t"<<"y="<<y1	<<endl;


	Kinematics Kin;
	Kin.init(sqrt(p.S),p.M);
	Kin.set_qT_y_get_ok(qT,y);
	Kin.update_zzb_and_xixib_get_OK(xx);

	Res_Kinematic_Parameters RKP;
	RKP.init(&Kin,p.prod,p.order,p.improved_pc(),p.order_as);
	RKP.set_qT(&Kin);
	RKP.set_y(&Kin);
	RKP.update_zz(&Kin);

	FO_Kinematic_Paramete_Set FKP;
	FKP.init(&Kin,p.order_as);
	FKP.set_qT(&Kin);
	FKP.set_y(&Kin);
	FKP.update_xx(&Kin);


	Quad_Differential_Crosssection QDCS1;
	Quad_Differential_Crosssection QDCS2(QDCS1);
	QDCS1.init(OP,pdfset[0],&RKP,&FKP,false,false);
	QDCS1.set_qT_mu(qT,&RKP,&FKP,mu,mu,1.);
	QDCS1.set_y<RES_MATCHED>(&RKP,&FKP);
	cout	<<"QDCS1<RES_MATCHED> = "<<QDCS1.update_xx<RES_MATCHED>(&RKP,&FKP,1.)<<endl<<endl;

	QDCS2.init(OP,pdfset[0],&RKP,&FKP,false,false);
	QDCS2.set_qT_mu(qT,&RKP,&FKP,mu,mu,1.);
	QDCS2.set_y<RES_MATCHED>(&RKP,&FKP);
	cout	<<"QDCS2<RES_MATCHED> = "<<QDCS2.update_xx<RES_MATCHED>(&RKP,&FKP,1.)<<endl<<endl;

	vector<Quad_Differential_Crosssection> V_QDCS;
	const int nQDCS(5);
	V_QDCS.resize(nQDCS);
	for(int i=0; i<nQDCS; ++i){
		V_QDCS[i].init(OP,pdfset[0],&RKP,&FKP,false,false);
		V_QDCS[i].set_qT_mu(qT,&RKP,&FKP,mu*(1.+i),mu,1.);
		V_QDCS[i].set_y<RES_MATCHED>(&RKP,&FKP);
		cout	<<"V_QDCS["<<i<<"]<RES_MATCHED> = "<<V_QDCS[i].update_xx<RES_MATCHED>(&RKP,&FKP,1.)<<endl;
	}
	cout	<<endl;

	QDCS1 = V_QDCS[3];
	cout	<<"QDCS1<RES_MATCHED> = "<<QDCS1.update_xx<RES_MATCHED>(&RKP,&FKP,1.)<<endl<<endl;
//	QDCS1.init(OP,pdfset[0],&RKP,&FKP,false,false);
	QDCS1.set_qT_mu(qT,&RKP,&FKP,mu,mu,1.);
	QDCS1.set_y<RES_MATCHED>(&RKP,&FKP);
	cout	<<"QDCS1<RES_MATCHED> = "<<QDCS1.update_xx<RES_MATCHED>(&RKP,&FKP,1.)<<endl<<endl;


	V_QDCS.assign(nQDCS,QDCS2);
	for(int i=0; i<nQDCS; ++i){
		V_QDCS[i].set_qT_mu(qT,&RKP,&FKP,mu*(1.+i),mu,1.);
		V_QDCS[i].set_y<RES_MATCHED>(&RKP,&FKP);
		cout	<<"V_QDCS["<<i<<"]<RES_MATCHED> = "<<V_QDCS[i].update_xx<RES_MATCHED>(&RKP,&FKP,1.)<<endl;
	}
	cout	<<endl;

}

inline void test_CS(SOPS_CuTe& OP){
	OP.Scheme.resummation.set(RES_MATCHED);
	NS_Utils::DOUT dout("test_IQI");
	double qT(10);
	double mu(10);
	double y(1);
	double xx[2];
	xx[0]=0.2; xx[1]=0.8;

	cout	<<OP<<endl;

	Crosssection CS;
	CS.init(OP);
	cout	<<"q* = "<<CS.get_q_star()<<endl;
	CS.update_qT(qT);
	CS.update_qT_y(qT,y);
//
//	using NS_Utils::operator<<;
//	cout	<< CS.results << endl;


}

//inline void test_barI2(){
//	NS_Utils::DOUT dout("test_barI2");
//	using namespace NS_gPDF;
//
//	double z	= 0.5;
//	double x	= 0.1;
//	double fz	= sin(z);
//	double f	= sin(1.);
////	double fz	= 1./z;
////	double f	= 1.;
//	int nf		= 5;
//
//	int order 		= 5;
//	bool improved	= true;
//
//	cout	<<endl	<<"Vars:" <<endl
//			<<"\tz="<<z	<<"\tx="<<x	<<"\tfz="<<fz	<<"\tf="<<f	<< endl
//			;
//	using NS_Ih::e_q;
//	using NS_Ih::e_g;
//	using NS_Ih::e_qb;
//	using NS_Ih::e_p;
//	using NS_gPDF::full_Ih_g;
//	using NS_gPDF::full_Ih_g2;
//	using NS_gPDF::full_Ih_q;
//
//	AD_Beta beta;
//	beta.select(nf);
//
//	NS_Ih::Ih_g_Params pg;
//	pg.init(order,improved);
//	pg.set_nf(nf);
//	pg.set_x(x);
//	pg.update(z);
//
//	cout 	<<"Glue:"<<endl;
//	cout	<<"barI_g_g:"<<endl<<"\t"<<full_Ih_g<e_g>(fz,f,pg) << endl;
//	cout	<<"barI_g_q:"<<endl<<"\t"<<full_Ih_g<e_q>(fz,f,pg) << endl;
//	cout	<<"barI_g2_g:"<<endl<<"\t"<<full_Ih_g2<e_g>(fz,f,pg) << endl;
//	cout	<<"barI_g2_q:"<<endl<<"\t"<<full_Ih_g2<e_q>(fz,f,pg) << endl;
//
//	NS_Ih::Ih_q_Params pq;
//	pq.init(order,improved);
//	pq.set_nf(nf);
//	pq.set_x(x);
//	pq.update(z);
//
//	cout 	<<"Quark:"<<endl;
//	cout	<<"barIq q:"<<endl<<"\t"<<full_Ih_q<e_q>(fz,f,pq) << endl;
//	cout	<<"barIq bq:"<<endl<<"\t"<<full_Ih_q<e_qb>(fz,f,pq) << endl;
//	cout	<<"barIq p:"<<endl<<"\t"<<full_Ih_q<e_p>(fz,f,pq) << endl;
//	cout	<<"barIq g:"<<endl<<"\t"<<full_Ih_q<e_g>(fz,f,pq) << endl;
//
//}
//
//
////inline void test_B(Options& op){
////	NS_Utils::DOUT dout("test_B");
////
////	PDFset pdfset;
////	pdfset.init(op);
////
////	double qT(10.);
////	double mu(15.);
////	double  y(0.);
////
////	CS_params<RES_LL> p_LL;
////	p_LL.init(op,pdfset[0]);
////	p_LL.update_qT_mu(qT,mu);
////	p_LL.update_y(y);
////
////	double z(0.5);
////
////	NS_gPDF::VectorbarI_H<RES_LL> I_H;
////	I_H.init(&p_LL,0);
////	I_H.set_xi();
//////	I_H.update_xi();
////	I_H.update_z(z);
////
////	cout	<<"I_H<"<<RES_LL<<">("<<p_LL.xi[0]<<","<<z<<"):"	<<endl
////			<<"\t"<<I_H[0]	<<endl
////			<<"\t"<<I_H[1]	<<endl;
//////	I_H<RES>(0,0.5):
//////		( ( 350.614) + ( -1531.7 + -1222.36*x^1)*x^1 + ( -22686.1 + -38403.4*x^1 + -57048.5*x^2)*x^2)
//////		( ( 0) + ( -3412.3)*x^1)
////
//////	I_H.update_y();
//////	cout	<<"I_H<"<<RES_LL<<">("<<p_LL.xi[0]<<"):"	<<endl
//////			<<"\t"<<I_H[0]	<<endl
//////			<<"\t"<<I_H[1]	<<endl;
//////	I_H<RES>(0):
//////		( ( 345.118) + ( -1486.09 + -1147.39*x^1)*x^1 + ( -34620.1 + -3801.78*x^1 + -15231.2*x^2)*x^2)
//////		( ( 0) + ( -3447.88)*x^1)
////
////
////	NS_gPDF::VectorbarI_DY<RES_LL> I_DY;
////	I_DY.init(&p_LL,0);
////	I_DY.set_xi();
//////	I_DY.update_xi();
////	I_DY.update_z(z);
////
////	cout	<<"I_DY<"<<RES_LL<<">("<<p_LL.xi[0]<<","<<z<<"):"	<<endl
////			<<"\t"<<I_DY[0]	<<endl
////			<<"\t"<<I_DY[1]	<<endl;
//////	I_DY<RES>(0,0.5):
//////		( ( 0))
//////		( ( 5.44621) + ( 101.278 + -93.8802*x^1)*x^1 + ( -1202.62 + -2052.35*x^1 + 423.799*x^2)*x^2)
////
//////	I_DY.update_y();
//////	cout	<<"I_DY<"<<RES_LL<<">("<<p_LL.xi[0]<<"):"	<<endl
//////			<<"\t"<<I_DY[0]	<<endl
//////			<<"\t"<<I_DY[1]	<<endl;
//////	I_DY<RES>(0):
//////		( ( 0))
//////		( ( 5.36084) + ( 59.8356 + -114.637*x^1)*x^1 + ( -2424.34 + -1225.1*x^1 + -541.724*x^2)*x^2)
////
////
////}
////
////inline void test_IQI(Options& op){
////	NS_Utils::DOUT dout("test_IQI");
////	PDFset pdfset;
////	pdfset.init(op);
////
////	double qT(10.);
////	double mu(15.);
////	double  y(0.);
////
////	CS_params<RES_LL> p_LL;
////	p_LL.init(op,pdfset[0]);
////	p_LL.update_qT_mu(qT,mu);
////	p_LL.update_y(y);
////
////	NS_gPDF::IQI<RES_LL> iqi_LL;
////	iqi_LL.init(&p_LL);
////
////	double zz[2];
////	zz[0]	= 0.1;
////	zz[1]	= 0.9;
////	Pol< Pol<double> > Erg;
////
////	iqi_LL.set_y();
////	iqi_LL.update_zz(zz,Erg);
////
////	cout	<<"iqi<"<<RES_LL<<">("<<y<<",["<<zz[0]<<","<<zz[1]<<"]):"	<<endl
////			<<"\t"<<Erg	<<endl;
//////	Neu:
//////	iqi<RES>(0,[0.1,0.9]):
//////		( ( 122930) + ( -13441 + 13008.4*x^1)*x^1 + ( -3035.22 + -8561.63*x^1 + 372.808*x^2)*x^2)
//////	ALt? Wie alt?
//////	iqi<RES>(0,[0.1,0.9]):
//////		( ( 122930) + ( -13441 + -14290.7*x^1)*x^1 + ( -3153.96 + 3971.21*x^1 + -160.065*x^2)*x^2)
////
//////	iqi_LL.update_y(Erg);
//////	cout	<<"iqi<"<<RES_LL<<">("<<y<<"):"	<<endl
//////			<<"\t"<<Erg	<<endl;
//////	iqi<RES>(0):
//////		( ( 119107) + ( -13138.5 + -10144*x^1)*x^1 + ( -1607.76 + 128.969*x^1 + -1508.8*x^2)*x^2)
////}
////
////inline void test_QDK(Options& op){
////	PDFset pdfset;
////	pdfset.init(op);
////
////	double qT(10.);
////	double mu(15.);
////	double  y(0.2);
////
////	CS_params<RES_LL> p_LL;
////	p_LL.init(op,pdfset[0]);
////	p_LL.update_qT_mu(qT,mu);
////	p_LL.update_y(y);
////
////	NS_RESUMED::Quad_Differential_Kernel<RES_LL> qdk_LL;
////	qdk_LL.init(&p_LL);
////	qdk_LL.set_qT_mu();
////
////	double zz[2];
////	zz[0]	= 0.1;
////	zz[1]	= 0.9;
////
////	qdk_LL.set_y();
////	qdk_LL.update_zz(zz);
////
////	cout	<<"qdk<"<<RES_LL<<">("<<y<<",["<<zz[0]<<","<<zz[1]<<"]):"	<<endl
////			<<"\t"<<qdk_LL	<<endl;
//////	Neu:
//////	H
//////	qdk<RES>(0.2,[0.1,0.9]):
//////		( 1.90654 + -0.621365*x^1 + 1.77925*x^2)
//////	Z
//////	qdk<RES>(0.2,[0.1,0.9]):
//////		( 0.0254586 + 0.0294257*x^1 + -0.108882*x^2)
////
////
//////	ALt? Wie alt?
////
//////	qdk<RES>(0.2,[0.1,0.9]):
//////		( 1.90654 + -0.111648*x^1 + -0.323508*x^2)
////
//////	qdk_LL.update_y();
//////	cout	<<"qdk<"<<RES_LL<<">("<<y<<"):"	<<endl
//////			<<"\t"<<qdk_LL	<<endl;
//////	qdk<RES>(0.2):
//////		( 429.35 + -30.7505*x^1 + -8.63565*x^2)
////}
//
////inline void test_QDCS(Options& op){
////
////	op.resummation.value=RES_MATCHED;
////
////	PDFset pdfset;
////	pdfset.init(op);
////
////	double qT(10.);
////	double mu(15.);
////	double  y(0.2);
////
////	NS_CROSSSECTION::Quad_Differential_Crosssection qdcs;
////	qdcs.init(op,pdfset[0]);
////	qdcs.set_qT_mu(qT,mu);
////	qdcs.set_y<RES_MATCHED>(y);
////
////	double zz[3];
////	zz[0]	= 0.1;
////	zz[1]	= 0.9;
////	zz[2]	= 0.4;
////	double erg;
////
////	erg = qdcs.update_xx<RES_MATCHED>(zz);
////	cout	<<"qdcs<"<<RES_MATCHED<<">(["<<zz[0]<<","<<zz[1]<<"])="<<erg	<<endl;
//////	qdcs<MATCHED>([0.1,0.9])=0.130618
////
////	erg = qdcs.update_xx<RES_LL>(zz);
////	cout	<<"qdcs<"<<RES_LL<<">(["<<zz[0]<<","<<zz[1]<<"])="<<erg	<<endl;
////}
//
//inline void test_DDCS(Options& op){
//
//	PDFset pdfset;
//	pdfset.init(op);
//
//	double qT(10.);
//	double  y(0.2);
//
//	Crosssection CS;
//	CS.init(op);
//	CS.update_qT_y(qT,y);
//
//	cout	<<"CS("<<qT<<","<<y<<")="<<CS.results	<<endl;
////	CS(10,0.2)=10 12.0819 0.2 0.139574 0.000138924 0 0 0 0 0 0
//}
//
//inline void test_qqbar(){
//	string filename("test.in");
//	Options op(filename);
//
//	PDFset pdfset;
//	pdfset.init(op);
//
//	NS_LHAFUNC::VectorPDF pdf1,pdf2;
//
//	pdf1.init(pdfset[0][0]);
//	pdf2.init(pdfset[0][1]);
//
//	NS_LHAFUNC::PartonVector<double> lumi,lumib;
//
//	double mu;
//	int N = op.qT().size();
//
//	double x[2];
//	x[0]=0.1;
//	x[1]=0.6;
//
//	NS_LHAFUNC::PartonVector<double>::iterator p1,p2,l1,l2;
//	NS_LHAFUNC::PartonVector<double>::reverse_iterator rp1,rp2,rl1,rl2;
//
//	for(int n=0; n<N; ++n){
//		mu	= op.qT()[n];
//
//		pdf1.update(x[0],mu);
//		pdf2.update(x[1],mu);
//
//		p1=pdf1.it_(1);
//		p2=pdf2.it_(1);
//		rp1=pdf1.rit_(-1);
//		rp2=pdf2.rit_(-1);
//
//		l1=lumi.it_(0);
//		l2=lumib.it_(0);
//		rl1=lumi.rit_(0);
//		rl2=lumib.rit_(0);
//
//		for(int i=0; i<6; ++i){
//			(*( ++l1))	= (* p1)*(* p2);
//			(*(++rl1))	= (*rp1)*(*rp2);
//
//			(*( ++l2))	= (* p1)*(*rp2);
//			(*(++rl2))	= (*rp1)*(* p2);
//			++p1;
//			++p2;
//			++rp1;
//			++rp2;
//		}
//
//		cout	<<"pdf1(x="<<x[0]<<",mu="<<mu<<")\t:"<<pdf1	<<endl
//				<<"pdf2(x="<<x[1]<<",mu="<<mu<<")\t:"<<pdf2	<<endl
//				<<"pdf1*pdf2     \t\t:"<<lumi	<<endl
//				<<"pdf1*bar(pdf2)\t\t:"<<lumib	<<endl
//				<<endl;
//	}
//
//
//}
//
//inline void testPlot(Options& op){
//	Plot plot;
//	plot.init(op);
//	plot.createPlot();
//}
//
//inline void test_MC(){
//	CuTe_MC::Test_Integrator::test();
//}
//
//
//
////inline void test_FO_H_param(Options& op){
////
////	double qT	=  0.1;
////	double mu	= 10.;
////	double  y	=  0.5;
////	int nf		= 5;
////
////	double xx[2];
////	xx[0]	= 0.2;
////	xx[1]	= 0.5;
////
////	op.resummation.value = RES_LO;
////	op.prod.value = PROD_H;
////
////	PDFset pdfset;
////	pdfset.init(op);
////
////	NS_CS_Params::CS_params<RES_LO> p;
////	p.init(op,pdfset[0]);
////	p.update_qT_mu(qT,mu);
////	p.update_y(y);
////
////	NS_H_NLO::Parameter HP,HP_tu;
////	HP.init(&p);
////	HP.set_qT_mu(qT,mu);
////	HP.set_y(+y);
////
////	HP.set_xi0(xx[0]);
////	HP.update_x2<2>(xx[1]);
////
////	HP.print_props();
////
////	HP_tu = HP;
////	HP_tu.swap_t_u();
////
////	NS_H_NLO::Parameter HP2,HP2_tu;
////	HP2.init(&p);
////	HP2.set_qT_mu(qT,mu);
////	HP2.set_y(-y);
////	HP2._set_xi0(HP.xi[1],1.);
////	HP2._set_xi1<2>(HP.xi[0],1.);
//////	HP2.swap_t_u();
////
////	HP2.print_props();
////
////	HP2_tu = HP2;
////	HP2_tu.swap_t_u();
////
////	HP2_tu.print_diff(HP);
////
////	NS_H_NLO::Parameter HP3,HP3_tu;
////	HP3.init(&p);
////	HP3.set_qT_mu(qT,mu);
////	HP3.set_y(-y);
////	HP3._set_xi0(HP.xi[1],1.);
////	HP3._set_x2<2>(HP.x2,1.);
////
////	HP3_tu = HP3;
////	HP3_tu.swap_t_u();
////
////	HP3_tu.print_diff(HP);
////
////	double tmp = NS_H_NLO::Coefficients::gg_2s_regular(HP,nf);
////
////	cout	<<"Delta gg_2s_regular(HP)="<<tmp	<<endl
////			<<"Delta gg_2s_regular(HP-HP2   )="<<(tmp-NS_H_NLO::Coefficients::gg_2s_regular(HP2,nf))	<<endl
////			<<"                              ="<<1./tmp*(tmp-NS_H_NLO::Coefficients::gg_2s_regular(HP2,nf))	<<endl
////			<<"Delta gg_2s_regular(HP-HP3   )="<<(tmp-NS_H_NLO::Coefficients::gg_2s_regular(HP3,nf))	<<endl
////			<<"                              ="<<1./tmp*(tmp-NS_H_NLO::Coefficients::gg_2s_regular(HP3,nf))	<<endl
////			<<endl;
////
////}
////
////
////
////inline void test_FO_H(Options& op){
////	op.resummation.value = RES_LO;
////	op.prod.value = PROD_H;
////
////	PDFset pdfset;
////	pdfset.init(op);
////
////
////	NS_CS_Params::CS_params<RES_LO> p;
////	p.init(op,pdfset[0]);
////
////	NS_CS_Params::CS_params<RES_LL> p_LL;
////	p_LL.init(op,pdfset[0]);
////
////	double qT	= 5;
////	double mu	= 30;
////	double y	= -1.85744;//0.;
////
////	double xx[2];
////	xx[0] = 0.5;
////	xx[1] = 0.5;
////
////	double xxy[3];
////	xxy[0] = 0.5;
////	xxy[1] = 0.5;
////	xxy[2] = 0.5;
////
////
////	p_LL.update_qT_mu(qT,mu);
////	p_LL.update_y(y);
////
////	NS_CROSSSECTION::Sigma_0 S0;
////	S0.init(&p_LL);
////	cout	<<"S0 = "<<S0.update_qT_mu()	<<"\t"<<"as = "<<p.a	<<endl;
////
////	NS_FIXED::QFO qfo;
////	qfo.init(&p);
////
////	p.update_qT_mu(qT,mu);
////	p.update_y(y);
////
////	qfo.set_qT_mu();
////	qfo.set_y();
////	qfo.update_xx(xx);
////
////
//////	p.update_qT_mu(qT,mu/2.);
//////	p.update_y(y);
//////
//////	qfo.set_qT_mu();
//////	qfo.set_y();
//////	qfo.update_xx(xx);
////
////
//////	p.update_qT_mu(qT,mu*2.);
//////	p.update_y(y);
//////
//////	qfo.set_qT_mu();
//////	qfo.set_y();
//////	qfo.update_xx(xx);
////
//////	Crosssection CS;
//////	CS.init(op);
//////	CS.update_qT_xxy(qT,xxy);
//////
//////	CS.update_qT(qT);
////
////}
//
//inline void test_for_delta_in_y(Options& op){
//
//	op.resummation.value = RES_MATCHED;
//	op.mu.value = 25.;
//	op.mu.is_set = true;
//
//	PDFset pdfset;
//	pdfset.init(op);
//
//
//
//	double xx1[2];
//	xx1[0]=0.01;
//	xx1[1]=0.99;
//
//	double xx2[2];
//	xx2[0]=xx1[1];
//	xx2[1]=xx1[0];
//
//	double y1 = 1.;
//	double y2 = -y1;
//	double qT= 20.;
//
////	Quad_Differential_Crosssection QDCS;
////	QDCS.init(op,pdfset[0]);
////	QDCS.set_qT_mu(qT,op.mu());
////
////	QDCS.set_y<RES_LLO>(y1);
////	cout	<<"QDCS(qT="<<qT<<",y="<<y1<<") = "<<QDCS.update_xx<RES_LLO>(xx1)<<endl;
////
////	QDCS.set_y<RES_LLO>(y2);
////	cout	<<"QDCS(qT="<<qT<<",y="<<y2<<") = "<<QDCS.update_xx<RES_LLO>(xx2)<<endl;
////
////
////	qT= 0.01;
////
////	QDCS.set_qT_mu(qT,op.mu());
////
////	QDCS.set_y<RES_LLO>(y1);
////	cout	<<"QDCS(qT="<<qT<<",y="<<y1<<") = "<<QDCS.update_xx<RES_LLO>(xx1)<<endl;
////
////	QDCS.set_y<RES_LLO>(y2);
////	cout	<<"QDCS(qT="<<qT<<",y="<<y2<<") = "<<QDCS.update_xx<RES_LLO>(xx2)<<endl;
//
//
//	Crosssection CS;
//	CS.init(op);
//	cout	<<"CS(qT="<<qT<<",y=+"<<y1<<") = "<<CS.update_qT_y(qT,y1)<<endl;
//	cout	<<"CS(qT="<<qT<<",y="<<y2<<") = "<<CS.update_qT_y(qT,y2)<<endl;
////	CS(qT=20,y=+1) = 20 25  1 0.164559 0.000124227 0.170776 0.149484 0 0 0 0	NZ0
////	CS(qT=20,y=+1) = 20 25  1 0.164557 0.00011995  0.170764 0.149499 0 0 0 0
////	CS(qT=20,y=+1) = 20 25  1 0.164554 0.000143537 0.170773 0.149477 0 0 0 0 	NZ0 NZ1
//
////	CS(qT=20,y=-1) = 20 25 -1 0.164555 0.000132565 0.170773 0.149481 0 0 0 0	NZ0
////	CS(qT=20,y=-1) = 20 25 -1 0.164547 0.000144978 0.170749 0.149498 0 0 0 0
////	CS(qT=20,y=-1) = 20 25 -1 0.164556 0.000116508 0.170775 0.149481 0 0 0 0 	NZ0 NZ1
//
//	qT	= 0.1;
//
//	cout	<<"CS(qT="<<qT<<",y=+"<<y1<<") = "<<CS.update_qT_y(qT,y1)<<endl;
////	CS(qT=0.1,y=+1) = 0.1 25  1 0.00204387 6.88353e-05 0.00828909 -0.00293233 0 0 0 0	NZ0
////	CS(qT=0.1,y=+1) = 0.1 25  1 0.00202723 0.000109653 0.00826714 -0.00295536 0 0 0 0
////	CS(qT=0.1,y=+1) = 0.1 25 1 -nan 2.22045e-16 -nan -nan 0 0 0 0  	NZ0 NZ1
////	CS(qT=0.1,y=+1) = 0.1 25 1 -nan 2.22045e-16 -nan -nan 0 0 0 0  	NZ1
//
//
//	cout	<<"CS(qT="<<qT<<",y="<<y2<<") = "<<CS.update_qT_y(qT,y2)<<endl;
////	CS(qT=0.1,y=-1) = 0.1 25 -1 0.00208867 5.828e-05 0.00833235 -0.00286398 0 0 0 0	NZ0
////	CS(qT=0.1,y=-1) = 0.1 25 -1 0.00209364 0.000102639 0.00833758 -0.00285995 0 0 0 0
////	CS(qT=0.1,y=-1) = 0.1 25 -1 -nan 2.22045e-16 -nan -nan 0 0 0 0  	NZ0 NZ1
////	CS(qT=0.1,y=-1) = 0.1 25 -1 -nan 2.22045e-16 -nan -nan 0 0 0 0  	NZ1
//}
//
//inline void test_Kin(Options& op){
//
//	op.resummation.value = RES_LL;
//	op.mu.value = 25.;
//	op.mu.is_set = true;
//
//	PDFset pdfset;
//	pdfset.init(op);
//
//	double qT	= 30;
//	double y	= 0.0;
//	double y1	= y;
//	double y2	= -y;
//
//
//	Crosssection CS;
//	CS.init(op);
//	cout	<<"CS_LL(qT="<<qT<<",y=+"<<y1<<") = "<<CS.update_qT_y(qT,y1)<<endl;
//	cout	<<"CS_LL(qT="<<qT<<",y=" <<y2<<") = "<<CS.update_qT_y(qT,y2)<<endl;
////	ONLY Q
////	(0.415605s)
////	CS_LL(qT=30,y=+0) = 30 25 0 0.000517754 4.82964e-08 0.0026072 -0.000510742 0 0 0 0
////	(0.479937s)
////	CS_LL(qT=30,y=-0) = 30 25 -0 0.000517754 4.82964e-08 0.0026072 -0.000510742 0 0 0 0
//
////	ONLY G
////	(0.366826s)
////	CS_LL(qT=30,y=+0) = 30 25 0 0.0823652 5.56538e-05 0.0889207 0.0677227 0 0 0 0
////	(0.362044s)
////	CS_LL(qT=30,y= -0) = 30 25 -0 0.0823652 5.56538e-05 0.0889207 0.0677227 0 0 0 0
//
//	op.resummation.value = RES_LO;
//	CS.init(op);
//	cout	<<"CS_LO(qT="<<qT<<",y=+"<<y1<<") = "<<CS.update_qT_y(qT,y1)<<endl;
//	cout	<<"CS_LO(qT="<<qT<<",y=" <<y2<<") = "<<CS.update_qT_y(qT,y2)<<endl;
////	ONLY Q
////	(1.99406s)
////	CS_LO(qT=30,y=+0) = 30 25 0 0.000461954 4.58934e-08 0.00307769 -0.000858653 0 0 0 0
////	(1.94956s)
////	CS_LO(qT=30,y=-0) = 30 25 -0 0.000461954 4.58934e-08 0.00307769 -0.000858653 0 0 0 0
//
////	ONLY G
////	(4.97709s)
////	CS_LO(qT=30,y=+0) = 30 25 0 0.132637 1.30824e-05 0.138332 0.118705 0 0 0 0
////	(5.02219s)
////	CS_LO(qT=30,y=-0) = 30 25 -0 0.132637 1.30824e-05 0.138332 0.118705 0 0 0 0
//
////	LO:
////	(0.078634s)
////	CS_LO(qT=30,y=+0) = 30 25 0 0.091463 1.47415e-06 0.127366 0.0664139 0 0 0 0
////	(0.0758059s)
////	CS_LO(qT=30,y=-0) = 30 25 -0 0.091463 1.47415e-06 0.127366 0.0664139 0 0 0 0
//
//
////	LO:	WITH_NEWX0
////	(0.0788491s)
////	CS_LO(qT=30,y=+0) = 30 25 0 0.0914632 7.41125e-06 0.127366 0.066414 0 0 0 0
////	(0.079077s)
////	CS_LO(qT=30,y=-0) = 30 25 -0 0.0914632 7.41125e-06 0.127366 0.066414 0 0 0 0
//
//
//	op.resummation.value = RES_MATCHING_CORR;
//	CS.init(op);
//	cout	<<"CS_MC(qT="<<qT<<",y=+"<<y1<<") = "<<CS.update_qT_y(qT,y1)<<endl;
//	cout	<<"CS_MC(qT="<<qT<<",y=" <<y2<<") = "<<CS.update_qT_y(qT,y2)<<endl;
////	ONLY Q
////	(18.7084s)
////	CS_MC(qT=30,y=+0) = 30 25 0 -5.62315e-05 4.11424e-09 -4.26533e-05 -5.67427e-05 0 0 0 0
////	(19.283s)
////	CS_MC(qT=30,y=-0) = 30 25 -0 -5.62315e-05 4.11424e-09 -4.26533e-05 -5.67427e-05 0 0 0 0
//
//
////	ONLY G
////	(16.1242s)
////	CS_MC(qT=30,y=+0) = 30 25 0 0.0456242 3.72388e-06 0.0479606 0.0420291 0 0 0 0
////	(15.7076s)
////	CS_MC(qT=30,y=-0) = 30 25 -0 0.0456242 3.72388e-06 0.0479606 0.0420291 0 0 0 0
//
////	LO:
////	(1.11285s)
////	CS_MC(qT=30,y=+0) = 30 25 0 0.0318496 2.74184e-06 0.0409568 0.0247684 0 0 0 0
////	(1.1275s)
////	CS_MC(qT=30,y=-0) = 30 25 -0 0.0318496 2.74184e-06 0.0409568 0.0247684 0 0 0 0
//
//
//
//}
//
//
//inline void test_gPDF(Options& op){
//	NS_TDPDF::TDPDFs B;
//	vector<double> results;
//
//	double xi=0.4;
//	double xT=1.;
//	double mu=10.;
//	double z=0.9;
//
//	B.init(op);
//
////
////	B.integrand(results,xi,xT,z);
////	B.print_results(cout,results);
////	cout<<endl;
//
//
//	B.integrate(results,xi,xT);
//	B.print_results(cout,results);
//	cout<<endl;
//
////
////	double xi=0.4;
////	double xT=1.;
////	double mu=10.;
////	double z=0.9;
////
////	B.init(op);
////
////	B.Ibar_pols_of_xi_mu_z(xi,mu,z);
////
////
////	cout<<"B(z="<<z<<",xT="<<xT<<")"<<endl;
////	B.Pols_to_Stream(cout);
////
////	B.Ibar_pols_of_xi_mu_xT_z(xi,mu,xT,z);
////
////	cout<<"B(z="<<z<<",xT="<<xT<<")"<<endl;
////	B.Pols_to_Stream(cout);
////
////	B.Ibar_pols_of_xi_mu_LT_z(xi,mu,B.LT,z);
////
////	cout<<"B(z="<<z<<",LT="<<B.LT<<")"<<endl;
////	B.Pols_to_Stream(cout);
////
////	B.Ibar_of_xi_mu_xT_z(xi,mu,xT,z);
////
////	cout<<"B(z="<<z<<",xT="<<xT<<")"<<endl;
////	B.Pols_to_Stream(cout);
////
////	B.Ibar_of_xi_mu_LT_z(xi,mu,B.LT,z);
////
////	cout<<"B(z="<<z<<",LT="<<B.LT<<")"<<endl;
////	B.Pols_to_Stream(cout);
////
////	B.Bbar_of_xi_mu_xT(xi,mu,xT);
////
////	cout<<"B(xT="<<xT<<")"<<endl;
////	B.Ints_to_Stream(cout);
////
////	vector<double> results;
////
////	NS_TDPDF::TDPDFs TD;
////	TD.init(op);
////	TD.Bbar_of_xi_xT(results,xi,xT);
////
////
//////	cout	<< results	<< endl;
////
////	TD.print_results(cout,results);
//
//}
//
//inline void test_epol(Options& op){
//
//	PDFset pdfset;
//	pdfset.init(op);
//
//	double qT	= 10.;
//	double  y	= 0.1;
//	double mu	= 15.;
//	double xxx[3];
//	xxx[0]	= 0.1;
//	xxx[1]	= 0.3;
//	xxx[2]	= 0.7;
//
//	NS_Kinematics::Kinematics Kin;
//	Kin.init(op);
//	Kin.set_qT(qT);
//	Kin.set_y(y);
//	Kin.update_zzb_xixib(xxx);
//
//
//
//	CS_params<RES_LL> p;
//	p.init(op,pdfset[0],&Kin);
//	p.update_qT_mu(Kin.qT,mu);
//	p.update_y(Kin.y);
//
//
//
//	NS_RESUMED::H_exp_Pol hpol;
//	hpol.init(&p);
//	NS_RESUMED::Anomaly_exp_Pol fpol;
//	fpol.init(&p);
//	NS_RESUMED::G_exp_Pol gpol;
//	gpol.init(&p);
//
//	Pol< complex<double> > Kexp;
//
////	cout	<<"   Epol.h2:"<<endl<<epol.h2()<<endl;
////	cout	<<"2*hpol(1.):"<<endl<<hpol.update_mu_a_l(1.).mul_by_scalar(2.)<<endl;
////	diff	= hpol.update_mu_a_l(1.).mul_by_scalar(2.).add_Pol(-epol.h2());
////	diff.reduce_order();
////	cout	<<"diff:"<<endl<<diff<<endl<<endl;
//
//	cout	<<gpol.properties()<<endl;
//
//
//	cout	<<"  hpol(as):"<<endl<<hpol.update_mu(Kexp)<<endl;
//	cout	<<"  fpol(as):"<<endl<<fpol.update_mu(Kexp)<<endl;
//	cout	<<endl;
//
//	cout	<<"  gpol(as):"<<endl<<gpol.update_mu(Kexp)<<endl;
//
//	cout	<<"exp(gpol(as)):"<<endl<<gpol.update_mu_exp(Kexp)<<endl;
//	cout	<<"Kexp:"<<endl<<Kexp<<endl;
//
//
//
//	p.update_qT_mu(Kin.qT,2.*mu);
//	p.update_y(Kin.y);
//
//
//	cout	<<gpol.properties()<<endl;
//
//	cout	<<"  hpol(as):"<<endl<<hpol.update_mu(Kexp)<<endl;
//	cout	<<"  fpol(as):"<<endl<<fpol.update_mu(Kexp)<<endl;
//	cout	<<endl;
//
//	cout	<<"  gpol(as):"<<endl<<gpol.update_mu(Kexp)<<endl;
//
//	cout	<<"exp(gpol(as)):"<<endl<<gpol.update_mu_exp(Kexp)<<endl;
//	cout	<<"Kexp:"<<endl<<Kexp<<endl;
//
//}
//
//inline void test_K_int(Options& op){
//
//	PDFset pdfset;
//	pdfset.init(op);
//	NS_Kinematics::Kinematics Kin;
//	Kin.init(op);
//
////
////	CS_params<RES_LL> p;
////	p.init(op,pdfset[0],&Kin);
////
////	Kin.set_qT(10.);
////	Kin.set_y(0.1);
////
////
////	NS_RESUMED::_K_INT_LL<RES_LL> K_old;
////	K_old.init(&p);
////	K_old.update();
////
////	NS_RESUMED::G_exp_Pol gexp;
////	gexp.init(&p);
////
////	NS_RESUMED::K_LL K_new;
////	K_new.init(gexp,&p);
////	K_new.update_mu();
////
////	cout	<<"g_exp:"<<endl<<K_old.barE()<<endl;
////	cout	<<"g_exp:"<<endl<<*(K_new.gexp)<<endl<<endl;
////
////	cout	<<"K_exp:"<<endl<<*(K_old.k_exp)<<endl;
////	cout	<<"K_exp:"<<endl<<K_new.kexp<<endl<<endl;
////
////	cout	<<"K_old:"<<endl<<K_old()<<endl;
////	cout	<<"K_new:"<<endl<<K_new._Kl<<endl<<endl;
////
////	Pol<Pol<double> > P(0,Pol<double>(0,1.));
////
////	cout	<<"K_old:"<<endl<<K_old.replaceL_E(P)<<endl;
////	cout	<<"K_new:"<<endl<<K_new.replaceL_E(P)<<endl;
//
//
//	CS_params<RES_LLO> p;
//	p.init(op,pdfset[0],&Kin);
//
//	p.update_qT_mu(10.,15.);
//
//	Kin.set_qT(10.);
//	Kin.set_y(0.1);
//
//
//
//	NS_RESUMED::G_exp_Pol gexp;
//	gexp.init(&p);
//
//	NS_RESUMED::K_LLO K_new;
//	K_new.init(gexp,&p);
//	K_new.update_mu();
//
//	cout	<<"g_exp:"<<endl<<*(K_new.gexp)<<endl<<endl;
//
////	cout	<<"K_exp:"<<endl<<*(K_old.k_exp)<<endl;
////	cout	<<"K_exp:"<<endl<<K_new.kexp<<endl<<endl;
//
//	cout	<<"K_old:"<<endl;
//	cout<<endl;
//	cout	<<"K_new:"<<endl;
//	K_new.print_Kl(cout);
//	cout<<endl<<endl;
//
//	Pol<Pol<double> > P(2);
//	for(int i=0; i<=P.order(); ++i){
//		P[i]=Pol<double>(2*i,1.);
//	}
//
//	cout	<<"K_new:"<<endl<<K_new.replaceL_E(P)<<endl;
//
//
//}
//
//inline void test_QDK(Options& op){
//
//	PDFset pdfset;
//	pdfset.init(op);
//
//	double qT	= 10.;
//	double  y	= 0.1;
//	double mu	= 15.;
//	double xxx[3];
//	xxx[0]	= 0.1;
//	xxx[1]	= 0.3;
//	xxx[2]	= 0.7;
//
//	NS_Kinematics::Kinematics Kin;
//	Kin.init(op);
//	Kin.set_qT(qT);
//	Kin.set_y(y);
//	Kin.update_zzb_xixib(xxx);
//
//
//
//	CS_params<RES_LL> p;
//	p.init(op,pdfset[0],&Kin);
//	p.update_qT_mu(Kin.qT,mu);
//	p.update_y(Kin.y);
//
//
//
//
//	NS_RESUMED::Quad_Differential_Kernel_new qdk_new;
//	qdk_new.init(&p);
//	qdk_new.set_qT_mu();
//	qdk_new.set_y();
//	qdk_new.update_zz(&Kin);
//
//	NS_RESUMED::Quad_Differential_Kernel_new qdk_new2(qdk_new);
//
//	cout	<<"qdk_new:"<<endl<<qdk_new<<endl;
//	cout	<<"qdk_new2:"<<endl<<qdk_new2<<endl<<endl;
//
//
//	p.update_qT_mu(Kin.qT,2.*mu);
//	p.update_y(Kin.y);
//
//
//	qdk_new.set_qT_mu();
//	qdk_new.set_y();
//	qdk_new.update_zz(&Kin);
//
//	qdk_new2.set_qT_mu();
//	qdk_new2.set_y();
//	qdk_new2.update_zz(&Kin);
//
//	cout	<<"qdk_new ="<<qdk_new.of_1()<<endl<<qdk_new<<endl;
//	cout	<<"qdk_new2="<<qdk_new2.of_1()<<endl<<qdk_new2<<endl<<endl;
//
//
//}
//
//
////inline void test_gPDFPlot(Options& op){
////	NS_gPDF::gPDF_Plot plot;
////	plot.init(op);
////
////	plot.plot();
////
////
////}

inline void test_OPs(){
	ofstream of;
	of.open("test_OPs.txt");

	stringstream ss;

	SOPS_CuTe OP("test.in");
	OP.to_ss(ss,"##\t");

	of	<< ss.str();
	of	<<endl<<endl;


	SOPS_CuTe op_new("test.in");

	op_new.into_stream(of,"##\t");

	of.close();


}
inline void test_qstar(SOPS_CuTe& OP){
	const double MZ(91.1876);
	const double MH(125.09);

	File_Path OutFileName(OP.get_infilename());
	cout	<<"Ifile: "<<OutFileName<<endl;
	OutFileName	= File_Path(OutFileName.dir,OutFileName.file.name,"qs.out");
	cout	<<"Ofile: "<<OutFileName<<endl;

	vector<string> PDFs;
	PDFs.push_back("MMHT2014nnlo68cl");
	PDFs.push_back("MMHT2014nlo68clas118");
	PDFs.push_back("MMHT2014nlo68cl");
	PDFs.push_back("MMHT2014lo68cl");

	PDFs.push_back("MSTW2008nnlo68cl");
	PDFs.push_back("MSTW2008nlo68cl");
	PDFs.push_back("MSTW2008lo68cl");

	PDFs.push_back("NNPDF23_nnlo_as_0118");
	PDFs.push_back("NNPDF23_nlo_as_0118");

	PDFs.push_back("NNPDF30_nnlo_as_0118");
	PDFs.push_back("NNPDF30_nlo_as_0118");
	PDFs.push_back("NNPDF30_lo_as_0118");

	PDFset pdfset;
	NS_CS_Params::Q_star QSZ;
	NS_CS_Params::Q_star QSH;

	ofstream of(OutFileName.full.c_str());

	of	<<"##";
	of	<<"\t"<<"MZ	= "<<MZ;
	of	<<"\t"<<"MH	= "<<MH;
	of	<<endl;

	of	<<"#";
	of	<<"\t"<<"PDF";
	of	<<"\t"<<"as(MZ)";
	of	<<"\t"<<"q*Z";
	of	<<"\t"<<"as(q*Z)";
	of	<<"\t"<<"as(MH)";
	of	<<"\t"<<"q*H";
	of	<<"\t"<<"as(q*H)";
	of	<<endl;


	for(int i=0; i<PDFs.size(); ++i){
		OP.PDF.PDFset.value	= PDFs[i];

		pdfset.init(OP);

		const NS_LHAFUNC::A_strong &as(*(pdfset.as_central()));

		QSZ.init(pdfset.as_central(),MZ,C_F*GAMMA_0);
		QSH.init(pdfset.as_central(),MH,C_A*GAMMA_0);

		of	<<PDFs[i];
		of	<<"\t"<<4.*M_PI*as.as(MZ);
		of	<<"\t"<<QSZ();
		of	<<"\t"<<4.*M_PI*as.as(QSZ());
		of	<<"\t"<<4.*M_PI*as.as(MH);
		of	<<"\t"<<QSH();
		of	<<"\t"<<4.*M_PI*as.as(QSH());
		of	<<	endl;


	}
	of.close();





}


inline void test(SOPS_CuTe& OP){
//	cout	<< op.properties()	<<endl;
//	test_barI2();
//	test_B(op);
//	test_IQI(op);
//	test_QDK(op);
//	test_QDCS(op);
//	test_DDCS(op);
	//		test_qqbar();

//	testPlot(op);

	//		test_MC();

	//		test_Bi();

//	test_FO_H(op);
//	test_FO_H_param(op);

//	test_for_delta_in_y(op);
//	test_Kin(op);
//	test_gPDF(op);
//	test_epol(op);
//	test_K_int(op);
//	test_QDK(op);
//	test_gPDFPlot(op);
//	test_Ibar_conv(op);
//	test_OPs();
//	test_K(OP);
//	test_Stepfunction(OP);
//	test_H(OP);
//	test_APSF(OP);
//	test_qstar(OP);
//	test_lnk(OP);
//	test_K(OP);
//	test_Rms2(OP);
//	test_K(OP);
//	test_IQI(OP);
//	test_QDCS(OP);
	test_CS(OP);
}

}  // namespace NS_test

#endif
#endif
