#ifndef IBAR_TEST_H
#define IBAR_TEST_H

#include "Ibar.h"
#include "TDPDF_options.h"

namespace CuTe {

namespace NS_Ibar {

using NS_TDPDF::SOPS_TDPDF;
using namespace NS_enums::NS_PC;

class Bbar: public Pol_a_l{
protected:
	NS_Ibar::Ibar_ij_conv_f Ibar;
	Ibar_G_Parameter IGP;
	Ibar_Q_Parameter IQP;
	NS_Ibar::Ibar_Parameter *IP;

	NS_CUBA_integration::cuba_integrand_t _integrand;
	NS_CUBA_integration::Cuba_Params cuba;

	double xi;
	int nf;
	double f1,fz;

	int a,b;


	inline double _fz(double z){
		return pow(z,a)*pow(1.-z,b);
	}

	inline void _set_xi_nf(double xi, int nf){
		this->xi	= xi;
		this->nf	= nf;
		f1	= _fz(xi/1.);
		IP->set_x(xi);
	}

	template<ENUM_IBAR_I i>
	inline void _set_integrand_i(e_P j){
		switch (j) {
		case e_g:
			_integrand	= _cuba_int_<i,e_g>;
			break;
		case e_q:
			_integrand	= _cuba_int_<i,e_q>;
			break;
		case e_qmp:
			_integrand	= _cuba_int_<i,e_qmp>;
			break;
		case e_qp:
			_integrand	= _cuba_int_<i,e_qp>;
			break;
//		case e_p:
//			_integrand	= _cuba_int_<i,e_p>;
//			break;
		case e_qbp:
			_integrand	= _cuba_int_<i,e_qbp>;
			break;
		case e_qbmp:
			_integrand	= _cuba_int_<i,e_qbmp>;
			break;
		case e_qb:
			_integrand	= _cuba_int_<i,e_qb>;
			break;
		default:
			break;
		}
	}


	inline void _set_integrand( ENUM_IBAR_I i, e_P j){
		switch (i) {
		case E_Ibar_Q:
			IP	= &IQP;
			_set_integrand_i<E_Ibar_Q>(j);
			break;
		case E_Ibar_G:
			IP	= &IGP;
			_set_integrand_i<E_Ibar_G>(j);
			break;
		default:
			IP	= &IGP;
			_set_integrand_i<E_Ibar_G2>(j);
			break;
		}
	}

	template<NS_Ibar::ENUM_IBAR_I B, NS_Ibar::e_P j>
	inline void _fill(double *ff,const double* xx){
		double Jac(1.-xi);
		double z= xi	+ Jac*(*xx);

		IP->update(z);
		fz	= _fz(xi/z);

		Ibar.set<B,j>(fz/z,f1/1.,IP,nf);
		for(int a=0; a<=IP->order; ++a){
			for(int l=0; l<=a; ++l){
				*(ff++)	= Jac*Ibar[a][l];
			}
		}
	}

	template<NS_Ibar::ENUM_IBAR_I B, NS_Ibar::e_P j>
	inline static int _cuba_int_(const int *ndim, const double xx[],const int *ncomp, double ff[], void *userdata){
		Bbar*	bbar	= static_cast<Bbar*>(userdata);
		bbar->_fill<B,j>(ff,xx);
		return 0;
	}

	inline static int calc_ncomp(int n){
		if(n==0) return 1;
		return n+1+calc_ncomp(n-1);
	}

	inline Pol_a_l& _set_from_cuba(){
//		NS_Utils::DOUT dout("Bbar::_set_from_cuba");
		int N(IP->order);
		order(N);

		std::vector<double>::const_iterator it(cuba.integral.begin());

//		using NS_Utils::operator<<;
//		dout()	<<"cuba.integral:\t"<<cuba.integral<<endl;

		for(int a=0; a<=N; ++a){
			_P[a].order(a);
			for(int l=0; l<=a; ++l){
				_P[a][l]	= *(it++);
			}
		}

		return *this;
	}

public:

	inline void init(const NS_CUBA_integration::OPS_CUBA &op_cuba){
		cuba.init(op_cuba);
	}

	inline void set(int order,POWER_COUNTING pc=PC_STD,int a=1, int b=1){
		const bool improved(pc==PC_IMP);
//		NS_Utils::DOUT dout("Bbar::set","order,a,b");
		IGP.init(order,improved);
		IQP.init(order,improved);
		this->a	= a;
		this->b	= b;

		cuba.set(1,calc_ncomp(order));

//		dout()	<<"order="<<order	<<"\t"<<"a="<<a	<<"\t"<<"b="<<b	<<"\t"<<"ncomp="<<calc_ncomp(order)	<<endl;

	}

	inline Pol_a_l& update(double xi, int nf, ENUM_IBAR_I i, e_P j){
//		NS_Utils::DOUT dout("Bbar::update","xi,nf,i,j");
//		dout()	<<"xi="<<xi	<<"\t"<<"nf="<<nf	<<"\t"<<"i="<<i	<<"\t"<<"j="<<j	<<endl;

		_set_integrand(i,j);
		_set_xi_nf(xi,nf);

		cuba.cube(_integrand,this);
//		cuba.test(1,_integrand,this);


		return _set_from_cuba();

	}

	inline const string to_string(int ntabs=0)const{
		stringstream ss;
		for(int n=0; n<ntabs; ++n) ss<<"\t";
		string tabs=ss.str();
		ss.str("");

		int N = _order;
		ss	<<tabs<<"( ";
		ss	<<"( "<<_P[0][0];
		ss	<<")*as^"<<0<<endl;


		for(int a=1; a<=N; ++a){
			ss	<<tabs<<"+ ";
			ss<<"( "<<_P[a][0];
			for(int l=1; l<=a; ++l){
				ss<<" + "<<_P[a][l]<<"*l^"<<l;
			}
			ss	<<")*as^"<<a<<endl;
		}
		ss	<<tabs<<")"<<endl;

		return ss.str();
	}

	inline const string to_string(double xi, int nf, ENUM_IBAR_I i, e_P j,int ntabs=0)const{
		stringstream ss;
		for(int n=0; n<ntabs; ++n) ss<<"\t";
		string tabs=ss.str();
		ss.str("");

		ss	<<tabs<<"[Ibar_"<<i<<"_"<<j<<" otimes (z)^"<<a<<"(1-z)^"<<b<<"](xi="<<xi<<",nf="<<nf<<"):"<<endl;
		ss	<<endl<<to_string(ntabs+1);
//		ss	<<endl<<IP->to_String(ntabs+1);

		return ss.str();
	}


};


inline void test(const SOPS_TDPDF &op){
	Bbar bbar;
	bbar.init(op.CUBA);

	int nf=5;

	if(!op.TDPDF.TDPDF_xi.is_set){
		std::cerr	<<"Set "<<op.TDPDF.TDPDF_xi.name<<" option"<<std::endl;
		return;
	}

	if(!op.PDF.PDF_Dummy_a_b.is_set){
		std::cerr	<<"Set "<<op.PDF.PDF_Dummy_a_b.name<<" option"<<std::endl;
		return;
	}

	std::vector<double>::const_iterator it(op.TDPDF.TDPDF_xi().begin());
	std::vector<double>::const_iterator end(op.TDPDF.TDPDF_xi().end());

	int order = op.Scheme.order();
	if(op.Scheme.Power_Counting()==PC_IMP){
		if(op.Scheme.order_epsilon.is_set){
			order	= op.Scheme.order_epsilon();
		}else{
			order	= 2*order +1;
		}
	}

	bbar.set(order,op.Scheme.Power_Counting(),op.PDF.PDF_Dummy_a_b()[0],op.PDF.PDF_Dummy_a_b()[1]);

	ENUM_IBAR_I i;
	e_P j;

	while(it!=end){
		i	= E_Ibar_G;

		j	= e_g;
		bbar.update(*it,nf,i,j);
		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

		j	= e_q;
		bbar.update(*it,nf,i,j);
		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

		cout	<<endl;
		i	= E_Ibar_G2;

		j	= e_g;
		bbar.update(*it,nf,i,j);
		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

		j	= e_q;
		bbar.update(*it,nf,i,j);
		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

		cout	<<endl;
		i	= E_Ibar_Q;

		j	= e_g;
		bbar.update(*it,nf,i,j);
		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

		j	= e_q;
		bbar.update(*it,nf,i,j);
		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

		j	= e_qmp;
		bbar.update(*it,nf,i,j);
		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

		j	= e_qp;
		bbar.update(*it,nf,i,j);
		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

//		j	= e_p;
//		bbar.update(*it,nf,i,j);
//		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

		j	= e_qbp;
		bbar.update(*it,nf,i,j);
		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

		j	= e_qbmp;
		bbar.update(*it,nf,i,j);
		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

		j	= e_qb;
		bbar.update(*it,nf,i,j);
		cout	<<bbar.to_string(*it,nf,i,j)<<endl;

		cout	<<endl<<"*****************************"<<endl<<endl;

		++it;
	}

}

}  // namespace NS_Ibar

}  // namespace CuTe


#endif //IBAR_TEST_H
